from typing import List, Dict

from ultipa.utils.convert import convertToGraph, convertTableToDict

from ultipa.configuration import RequestConfig

from ultipa.structs import HDCUpdateType, GraphSet
from ultipa.utils import  CommandList,UQLMAKER
from ultipa.types import ULTIPA, ULTIPA_RESPONSE
from ultipa.operate.base_extra import BaseExtra
from ultipa.structs.Schema import Schema
REPLACE_KEYS = {
	"graph": "name",
}
class HDCExtra(BaseExtra):

    def createHDCGraphBySchema(self,graphName: str,nodeSchemas: [Dict[str,List[str]]],edgeSchemas:[Dict[str,List[str]]],hdcName: str,updata: HDCUpdateType,requestConfig: RequestConfig = RequestConfig())->ULTIPA_RESPONSE.UltipaResponse:
        command=CommandList.createHDCGraph
        uqlmarker=UQLMAKER(command=command,commonParams=requestConfig)
        def getschemalist(schemas: []):
            schemalist = []
            for data in schemas:
                for key,value in data.items():
                    if key== '*':
                        schemalist.append(f'"{key}":{value}')
                    else :
                        schemalist.append(f'{key}:{value}')
            return schemalist

        nodeSchemas = ','.join(getschemalist(nodeSchemas))
        edgeSchemas = ','.join(getschemalist(edgeSchemas))
        commP = '{nodes:{%s},edges:{%s},query:"query",type:"Graph",update:"%s",default:true}' % (
        nodeSchemas, edgeSchemas,updata)

        uqlmarker.setCommandParams([graphName,commP])
        uqlmarker.addParam(key='to', value=f'{hdcName}')
        res = self.uqlSingle(uqlmarker)
        return res
    def showHDCGraph(self,requestConfig: RequestConfig = RequestConfig())->List[dict]:

        command=CommandList.showHDCGraph
        uqlmaker=UQLMAKER(command=command , commonParams=requestConfig)
        uqlmaker.setCommandParams("")
        res = self.uqlSingle(uqlmaker)
        if res.status.code == 0:
            if res.alias(f"_projectList").data.rows:
                result = convertTableToDict(res.alias("_projectList").data.rows,res.alias("_projectList").data.headers)
                return result
        return res

    def dropHDCGraph(self,graphName:str,requestConfig: RequestConfig = RequestConfig())->ULTIPA_RESPONSE.UltipaResponse:
        command=CommandList.dropHDCGraph
        uqlmaker = UQLMAKER(command=command, commonParams=requestConfig)
        uqlmaker.setCommandParams(graphName)
        res = self.uqlSingle(uqlmaker)
        return res




from typing import List

from ultipa import DBType
from ultipa.utils.convert import convertTableToDict
from ultipa.structs.Job import Job
from ultipa.operate.base_extra import BaseExtra
from ultipa.utils import UQLMAKER, CommandList
from ultipa.types import ULTIPA_REQUEST, ULTIPA, ULTIPA_RESPONSE
from ultipa.configuration.RequestConfig import RequestConfig

JSONSTRING_KEYS = ["graph_privileges", "system_privileges", "policies", "policy", "privilege"]
formatdata = ['graph_privileges']


class TruncateExtra(BaseExtra):
    
	'''
        Processing class that defines settings for advanced operations on graphset.
	'''

	def truncate(self, request: ULTIPA_REQUEST.Truncate,
				 requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Truncate graphset.

		Args:
			request: An object of Truncate class

			requestConfig: An object of RequestConfig class

		Returns:
			Response

		'''
		uqlResponse=ULTIPA_RESPONSE.Response()
		command = CommandList.truncate
		requestConfig.graphName = request.graphSetName

		if request.dbType is None and request.schema is not None:
				uqlResponse.status = ULTIPA.Status(code=ULTIPA.Code.UQL_ERROR, message="To truncate schema, DbType is required in the parameters")
				return uqlResponse

		uqlMaker = UQLMAKER(command, commonParams=requestConfig)
		uqlMaker.addParam("graph", request.graphSetName)

		if request.dbType is not None:
			if request.dbType == DBType.DBNODE:
				if request.schema=="*" or request.schema is None:
					uqlMaker.addParam("nodes", "*")
				else:
					uqlMaker.addParam("nodes", "@" + request.schema, notQuotes=True)
			if request.dbType == DBType.DBEDGE:
				if request.schema=="*" or request.schema is None:
					uqlMaker.addParam("edges", "*")
				else:
					uqlMaker.addParam("edges", "@" + request.schema, notQuotes=True)

		# if request.all and not request.dbType:
		#     uqlMaker = UQLMAKER(command,commandP=request.dbType,commonParams=requestConfig)

		return self.uqlSingle(uqlMaker)

	def compact(self, graphName: str,
				requestConfig: RequestConfig = RequestConfig()) -> Job:
		'''
		Compact graphshet.

		Args:
			graph: The name of graphset

			requestConfig: An object of RequestConfig class

		Returns:
			ResponseCommon

		'''
		command = CommandList.compact
		uqlMaker = UQLMAKER(command, commonParams=requestConfig)
		uqlMaker.addParam("graph", graphName)
		result=self.uqlSingle(uqlMaker)
		if result.items:
			res = convertTableToDict(result.alias("result").data.rows, result.alias("result").data.headers)
			return Job(id=res[0]['new_job_id'])
		else :
			return Job(id='',error=result.status.message)


	def mountGraph(self, graphName: str,
				   requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Mount graphshet.

		Args:
			graph: The name of graphset

			requestConfig: An object of RequestConfig class

		Returns:
			Response

		'''
		commonP = []
		if graphName:
			commonP = graphName
			requestConfig.graphName = graphName
		uqlMaker = UQLMAKER(command=CommandList.mount, commonParams=requestConfig)
		uqlMaker.setCommandParams(commandP=commonP)
		return self.uqlSingle(uqlMaker)

	def unmountGraph(self, graphName: str,
					 requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Unmount graphshet.

		Args:
			graph: The name of graphset

			requestConfig: An object of RequestConfig class

		Returns:
			Response

		'''
		commonP = []
		if graphName:
			commonP = graphName
			requestConfig.graphName = graphName
		uqlMaker = UQLMAKER(command=CommandList.unmount, commonParams=requestConfig)
		uqlMaker.setCommandParams(commandP=commonP)
		return self.uqlSingle(uqlMaker)



from ultipa.configuration.RequestConfig import RequestConfig
from ultipa.operate.base_extra import BaseExtra
from ultipa.structs import DBType
from ultipa.structs.Property import Property
from ultipa.types import ULTIPA, ULTIPA_RESPONSE
from ultipa.utils import UQLMAKER, CommandList
from ultipa.utils.ResposeFormat import ResponseKeyFormat
from ultipa.utils.propertyUtils import getPropertyTypesDesc
from typing import Tuple, List, Dict

BOOL_KEYS = ["index", "lte"]
REPLACE_KEYS = {
	"name": "propertyName",
	"type": "propertyType",
}


class PropertyExtra(BaseExtra):
	'''
	Processing class that defines settings for property related operations.
	'''

	# def listProperty(self, requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.ResponseListProperty:
	# 	'''
	# 	List all properties.

	# 	Args:
	# 		requestConfig: An object of RequestConfig class

	# 	Returns:
	# 		ResponseListProperty

	# 	'''
	# 	return self.showProperty(requestConfig)

	def showProperty(self, dbType:DBType = None, schemaName: str = None,
					 requestConfig: RequestConfig = RequestConfig()) -> Dict[str,List[Property]]:
		'''
		Show all Node or Edge Schema properties.

		Args:
			dbType: The DBType of data (DBNODE or DBEDGE)

			schema: The name of schema

			requestConfig: An object of RequestConfig class

		Returns:
			ResponseListProperty

		'''
		if dbType != None:
			if dbType == DBType.DBNODE:
					command = CommandList.showNodeProperty
			elif dbType == DBType.DBEDGE:
					command = CommandList.showEdgeProperty
			else:
				raise TypeError("dbType must be an instance of DBType.DBNODE or DBType.DBEDGE")
			if schemaName:
				commandp = ['@' + f"`{schemaName}`"]
			else:
				commandp = ''
		else:
			command = CommandList.showProperty
			commandp = ''

		uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
		uqlMaker.setCommandParams(commandP=commandp)
		# res = self.UqlListSimple(uqlMaker=uqlMaker,
		# 						 responseKeyFormat=ResponseKeyFormat(boolKeys=BOOL_KEYS),
		# 						 isSingleOne=False)
		res = self.uqlSingle(uqlMaker=uqlMaker)
		propertyDict = {}
		nodeProperty = res.alias('_nodeProperty').asProperties()
		edgeProeprty = res.alias('_edgeProperty').asProperties()
		if nodeProperty:
			propertyDict['_nodeProperty'] = nodeProperty
		if edgeProeprty:
			propertyDict['_edgeProperty'] = edgeProeprty


		return propertyDict
	def showNodeProperty(self, schemaName: str = None,
						 requestConfig: RequestConfig = RequestConfig()) -> List[Property]:
		'''
		Show all Node Schema properties.

		Args:
			schemaName: The name of schema

			requestConfig: An object of RequestConfig class

		Returns:
			List[Property]

		'''
		res = self.showProperty(schemaName=schemaName, dbType=DBType.DBNODE, requestConfig=requestConfig)
		return res.get('_nodeProperty')

	def showEdgeProperty(self, schemaName: str = None,
						 requestConfig: RequestConfig = RequestConfig()) -> List[Property]:
		'''
		Show all Edge Schema properties.

		Args:
			schemaName: The name of schema

			requestConfig: An object of RequestConfig class

		Returns:
			List[Property]

		'''

		res = self.showProperty(schemaName=schemaName, dbType=DBType.DBEDGE, requestConfig=requestConfig)
		return res.get('_edgeProperty')

	def createProperty(self, dbType: DBType, schemaName: str, prop: Property,
					   requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Create a property.

		Args:
			dbType: The DBType of data (DBNODE or DBEDGE)

			schema: The name of schema

			prop:  An object of Property class

			requestConfig: An object of RequestConfig class

		Returns:
			UltipaResponse

		'''
		command = dbType == DBType.DBNODE and CommandList.createNodeProperty or CommandList.createEdgeProperty
		commandP = ["@" + f"`{schemaName}`", f"`{prop.name}`",
					getPropertyTypesDesc(prop.type, prop.subTypes)]

		if prop.description:
			commandP.append(prop.description)
		uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
		uqlMaker.setCommandParams(commandP=commandP)
		res = self.uqlSingle(uqlMaker)
		return res

	def dropProperty(self, dbType: DBType, schemaName: str, propertyName: str,
					 requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Drop a property.

		Args:
			dbType: The DBType of data (DBNODE or DBEDGE)

			schema: The name of schema

			property: The name of property

			requestConfig: An object of RequestConfig class

		Returns:
			UltipaResponse

		'''
		command = dbType == DBType.DBNODE and CommandList.dropNodeProperty or CommandList.dropEdgeProperty
		commandP = "@`%s`.`%s`" % (schemaName, propertyName)
		uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
		uqlMaker.setCommandParams(commandP=commandP)
		res = self.uqlSingle(uqlMaker)
		return res

	def alterProperty(self, dbType: DBType, property: Property, newProperty: Property,
					  requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Alter a property.

		Args:
			dbType: The DBType of data (DBNODE or DBEDGE)

			property: The name of property

			newProperty: The new name of property

			requestConfig: An object of RequestConfig class

		Returns:
			UltipaResponse

		'''
		command = dbType == DBType.DBNODE and CommandList.alterNodeProperty or CommandList.alterEdgeProperty
		commandP = "@`%s`.`%s`" % (property.schema, property.name)
		update_dict = {}
		if newProperty.name:
			update_dict.setdefault('name', newProperty.name)
		if newProperty.description:
			update_dict.update({'description': newProperty.description})
		uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
		uqlMaker.setCommandParams(commandP)
		uqlMaker.addParam("set", update_dict)
		res = self.uqlSingle(uqlMaker)
		return res



import time

import json
from typing import Tuple, List

from ultipa.configuration.RequestConfig import RequestConfig
from ultipa.operate.base_extra import BaseExtra
from ultipa.structs import Graph
from ultipa.types import ULTIPA, ULTIPA_RESPONSE
from ultipa.types.types import Status, Code
from ultipa.utils import UQLMAKER, CommandList
from ultipa.utils.ResposeFormat import ResponseKeyFormat
from ultipa.utils.convert import convertToGraph
from ultipa.structs.GraphSet import GraphSet
REPLACE_KEYS = {
	"graph": "name",
}


class GraphExtra(BaseExtra):
	'''
	Processing class that defines settings for graphset related operations.
	'''

	def uqlCreateSubgraph(self, uql: str, subGraphName: str, requestConfig: RequestConfig = RequestConfig()):
		ret = self.uql(uql, requestConfig)
		graphRet = self.getGraph(subGraphName)
		if graphRet:
			self.createGraph(GraphSet(name=subGraphName))
			time.sleep(3)

	def showGraph(self, requestConfig: RequestConfig = RequestConfig()) ->List[GraphSet]:
		'''
		Acquire graphset list.

		Args:
			requestConfig: An object of RequestConfig class

		Returns:
			List[GraphSet]
		'''

		uqlMaker = UQLMAKER(command=CommandList.showGraphMore, commonParams=requestConfig)
		uqlMaker.setCommandParams("")
		res = self.UqlListSimple(uqlMaker=uqlMaker, responseKeyFormat=ResponseKeyFormat(keyReplace=REPLACE_KEYS))
		if res.status.code ==0 :
			res.data = convertToGraph(res)
			return res.data
		return ULTIPA_RESPONSE.UltipaResponse(status=res.status)
		# uqlMaker = UQLMAKER(command=CommandList.showGraph, commonParams=requestConfig)
		# uqlMaker.setCommandParams("")
		# res = self.UqlListSimple(uqlMaker=uqlMaker, responseKeyFormat=ResponseKeyFormat(keyReplace=REPLACE_KEYS))
		# res.data = convertToGraph(res)
		# return res.data

	def getGraph(self, graphName: str,
				 requestConfig: RequestConfig = RequestConfig()) -> GraphSet:
		'''
		Acquire a designated graphset.

		Args:
			graphName: The name of graphset

			requestConfig: An object of RequestConfig class

		Returns:
			GraphSet
		'''

		uqlMaker = UQLMAKER(command=CommandList.showGraph, commonParams=requestConfig)
		uqlMaker.setCommandParams(graphName)
		uqlMaker.addParam(key='more',value=graphName)
		res = self.UqlListSimple(uqlMaker=uqlMaker, responseKeyFormat=ResponseKeyFormat(keyReplace=REPLACE_KEYS))
		if res.status.code == ULTIPA.Code.SUCCESS:
			res.data = convertToGraph(res)
		else:
			res.data = None
		return res.data

	def createGraph(self, graph: GraphSet,
					config: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Create a graphset.

		Args:
			grpah: An object of Graph class

			config: An object of RequestConfig class

		Returns:
			UltipaResponse
		'''
		uqlMaker = UQLMAKER(command=CommandList.createGraph, commonParams=config)
		if graph.shards and  graph.partitionBy:
			shardslist=[int(shard) for shard in graph.shards]
			if graph.description:
				uqlMaker.setCommandParams([graph.name, graph.description])

			else:
				uqlMaker.setCommandParams(graph.name)
			uqlMaker.addParam(key='shards', value=shardslist)
			uqlMaker.addParam(key='partitionByHash', value=f"'{graph.partitionBy}',_id")
			res = self.uqlSingle(uqlMaker)
			return res
		else:
			message = 'partitionBy and shards is empty' if not graph.shards and  not graph.partitionBy else 'shards is empty' if not graph.shards else 'partitionBy is empty'

			return ULTIPA_RESPONSE.UltipaResponse(status=Status(code = Code.FAILED,message=message))


	def dropGraph(self, graphName: str,
				  requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Drop a graphset.

		Args:
			graphName: The name of graphset

			requestConfig: An object of RequestConfig class

		Returns:
			UltipaResponse
		'''

		uqlMaker = UQLMAKER(command=CommandList.dropGraph, commonParams=requestConfig)
		uqlMaker.setCommandParams(graphName)
		res = self.uqlSingle(uqlMaker)
		return res

	def alterGraph(self, oldGraph: GraphSet, newGraph: GraphSet,
			   requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		        Alter a graphset

		        Args:
		            oldGraphName: The orignal name of graphset

		            newGraphName: The new name of graphset

		            newDescription: The new description of graphset

		            requestConfig: An object of RequestConfig class

		        Returns:
		            UltipaResponse
		        '''

		requestConfig.graphName = oldGraph.name
		uqlMaker = UQLMAKER(command=CommandList.alterGraph, commonParams=requestConfig)
		uqlMaker.setCommandParams(oldGraph.name)
		data = {"name": newGraph.name}
		if newGraph.description is not None:
			data.update({'description': newGraph.description})
		uqlMaker.addParam("set", data)
		res = self.uqlSingle(uqlMaker)
		return res

	def hasGraph(self, graphName: str, requestConfig: RequestConfig = RequestConfig()) -> bool:
		'''
				Check if graph exists or not.

				Args:
					graphName: The name of graphset

					requestConfig: An object of RequestConfig class

				Returns:
					bool
				'''

		graphsdata = self.showGraph(requestConfig)
		for graphs in graphsdata:
			if (graphs.name == graphName):
				return True

		return False

	def createGraphIfNotExist(self, graph: GraphSet,
							  config: RequestConfig = RequestConfig()) -> Tuple[
		bool, ULTIPA_RESPONSE.UltipaResponse]:
		'''
		Checks if graph exists or not, if graph does not exist then creates new.

		Args:
			graphName: The name of graphset

			requestConfig: An object of RequestConfig class

		Returns:
			bool
			UltipaResponse
		'''

		if (self.hasGraph(graph.name, config)) == True:
			return [True, None]

		else:
			res = self.createGraph(graph,config = config)
			return [False, res]

	def rebalanceGraph(self,name: str,shards:List[str],
					   requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		"""
		graph migrate
		name: The name of graph
		shards: The shards of new shards
		"""
		shardslist = [int(shard) for shard in shards]
		uqlMaker = UQLMAKER(command=CommandList.rebalanceGraph, commonParams=requestConfig)
		uqlMaker.setCommandParams(name)
		uqlMaker.addParam(key='shards', value=shardslist)
		res = self.uqlSingle(uqlMaker)
		return res


# -*- coding: utf-8 -*-
# @Time    : 2023/8/1 10:47
# @Author  : Ultipa
# @Email   : support@ultipa.com
# @File    : Graph.py
from typing import List

from ultipa.structs.BaseModel import BaseModel

class GraphSet(BaseModel):


    def __init__(self, name: str,shards: List[str] = None, partitionBy: str = None, id: int = None, totalNodes: int = None,
                 totalEdges: int = None, description: str = None,slotNum :str = '', status: str = None):
        self.id = id
        self.name = name
        self.totalNodes = totalNodes
        self.totalEdges = totalEdges
        self.description = description
        self.status = status
        self.shards = shards
        self.slotNum = slotNum
        self.partitionBy = partitionBy






# -*- coding: utf-8 -*-
# @Time    : 2024/05/14 11:47
# @Author  : Ultipa
# @Email   : support@ultipa.com
# @File    : Index.py
from typing import List

from ultipa.structs import DBType
from ultipa.structs.BaseModel import BaseModel


class Process(BaseModel):
    '''
        Data class for Index.
    '''

    def __init__(self, id: str,
                 uql: str,
                 gql: str,
                 duration: int,
                 status: str):
        self.id = id
        self.type = type
        self.uql = uql
        self.gql = gql
        self.duration = duration
        self.status = status




from ultipa.types.types_response import Privilege

from ultipa.configuration.RequestConfig import RequestConfig
from ultipa.operate.base_extra import BaseExtra
from ultipa.types import  ULTIPA_RESPONSE, ULTIPA
from ultipa.utils import UQLMAKER, CommandList
from ultipa.utils.errors import ParameterException
from ultipa.utils.ResposeFormat import ResponseKeyFormat
from typing import List
from ultipa.structs.Policy import Policy
from ultipa.utils.convert import convertToPolicy
from ultipa.utils.convert import convertToPrivilege

JSONSTRING_KEYS = ["graphPrivileges", "systemPrivileges", "policies", "policy", "privilege"]
formatdata = ['graph_privileges']


class PolicyExtra(BaseExtra):
    '''
        Processing class that defines settings for policy related operations.
    '''

    def showPolicy(self,
                   requestConfig: RequestConfig = RequestConfig()) -> List[Policy]:
        '''
        Show policy list.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Policy]

        '''
        uqlMaker = UQLMAKER(command=CommandList.showPolicy, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker,
                                 responseKeyFormat=ResponseKeyFormat(jsonKeys=JSONSTRING_KEYS, dataFormat=formatdata))
        if len(res.data) > 0:
            res.data = convertToPolicy(res)
        return res.data

    def showPrivilege(self,
                      requestConfig: RequestConfig = RequestConfig()) -> Privilege:
        '''
        Show privilege list.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            Privilege

        '''

        uqlMaker = UQLMAKER(command=CommandList.showPrivilege, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker,
                                 responseKeyFormat=ResponseKeyFormat(jsonKeys=JSONSTRING_KEYS))
        if len(res.data) > 0:
            res.data = convertToPrivilege(res)
        return res.data

    def getPolicy(self, policyName: str,
                  requestConfig: RequestConfig = RequestConfig()) -> Policy:
        '''
        Get a policy.

        Args:
            name: The name of policy

            requestConfig: An object of RequestConfig class

        Returns:
            Policy

        '''

        uqlMaker = UQLMAKER(command=CommandList.getPolicy, commonParams=requestConfig)
        uqlMaker.setCommandParams(policyName)
        res = self.UqlListSimple(uqlMaker=uqlMaker,
                                 responseKeyFormat=ResponseKeyFormat(jsonKeys=JSONSTRING_KEYS, dataFormat=formatdata))
        # if isinstance(res.data, list) and len(res.data) > 0:
        # 	res.data = res.data[0]
        if res.status.code == ULTIPA.Code.SUCCESS and res.data:
            res.data = convertToPolicy(res)
        return res.data

    def createPolicy(self, policy: Policy,
                     requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Create a policy.

        Args:
            request:  An object of Policy class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''

        uqlMaker = UQLMAKER(command=CommandList.createPolicy, commonParams=requestConfig)
        paramsP = [policy.name]
        if policy.graphPrivileges:
            paramsP.append(policy.graphPrivileges)
        else:
            paramsP.append({})

        if policy.systemPrivileges:
            paramsP.append(policy.systemPrivileges)
        else:
            paramsP.append([])

        if policy.policies:
            paramsP.append(policy.policies)
        else:
            paramsP.append([])

        if policy.propertyPrivileges:
            paramsP.append(policy.propertyPrivileges)
        else:
            paramsP.append({})

        uqlMaker.setCommandParams(paramsP)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res

    def alterPolicy(self, policy: Policy,
                    requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Alter a policy.

        Args:
            request:  An object of Policy class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''

        uqlMaker = UQLMAKER(command=CommandList.alterPolicy, commonParams=requestConfig)
        uqlMaker.setCommandParams(policy.name)
        params = {}
        if policy.systemPrivileges is not None:
            params.update({"system_privileges": policy.systemPrivileges})
        if policy.graphPrivileges is not None:
            params.update({"graph_privileges": policy.graphPrivileges})

        if policy.policies is not None:
            params.update({"policies": policy.policies})

        if policy.propertyPrivileges is not None:
            params.update({"property_privileges": policy.propertyPrivileges})

        uqlMaker.addParam('set', params, notQuotes=True)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res

    def dropPolicy(self, policyName: str,
                   requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Drop a policy.

        Args:
            policyName:  Name of policy to be dropped
            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''

        uqlMaker = UQLMAKER(command=CommandList.dropPolicy, commonParams=requestConfig)
        uqlMaker.setCommandParams(policyName)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res

    def grantPolicy(self, userName: str = '', graphPrivileges: dict = None,
                    systemPrivileges: List[str] = None, policies: List[str] = None, propertyPrivileges: dict = None,
                    requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Grant privileges and policies to a user.

        Args:
            username (str): The username to grant privileges and policies.

            graphPrivileges (dict): Dictionary containing graph privileges.

            systemPrivileges (List[str]): List of system privileges.

            policies (List[str]): List of policies to grant.

            propertyPrivileges(dict): Dictionary containing property privileges

            requestConfig (RequestConfig): An object of RequestConfig class

        Returns:
            UltipaResponse

        '''

        uqlMaker = UQLMAKER(command=CommandList.grantUser, commonParams=requestConfig)
        if userName:
            uqlMaker.setCommandParams(userName)
        else:
            raise ParameterException(err='username is a required parameter')

        paramsDict = {}
        if graphPrivileges:
            paramsDict.setdefault('graph_privileges', graphPrivileges)

        if systemPrivileges:
            paramsDict.setdefault('system_privileges', systemPrivileges)

        if policies:
            paramsDict.setdefault('policies', policies)

        if propertyPrivileges:
            paramsDict.setdefault('property_privileges', propertyPrivileges)

        uqlMaker.addParam('params', paramsDict, notQuotes=True)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res

    def revokePolicy(self, userName: str = '', graphPrivileges: dict = None,
                     systemPrivileges: List[str] = None, policies: List[str] = None, propertyPrivileges: dict = None,
                     requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Revoke privileges and policies from a user.

        Args:
            username (str): The username to revoke privileges and policies.

            graphPrivileges (dict): Dictionary containing graph privileges.

            systemPrivileges (List[str]): List of system privileges.

            propertyPrivileges(dict): Dictionary containing property privileges

            policies (List[str]): List of policies to revoke.

            requestConfig (RequestConfig): An object of RequestConfig class

        Returns:
            UltipaResponse

        '''

        uqlMaker = UQLMAKER(command=CommandList.revoke, commonParams=requestConfig)
        if userName:
            uqlMaker.setCommandParams(userName)
        else:
            raise ParameterException(err='username is a required parameter')

        paramsDict = {}
        if graphPrivileges:
            paramsDict.setdefault('graph_privileges', graphPrivileges)

        if systemPrivileges:
            paramsDict.setdefault('system_privileges', systemPrivileges)

        if propertyPrivileges:
            paramsDict.setdefault('property_privileges', propertyPrivileges)

        if policies:
            paramsDict.setdefault('policies', policies)
        uqlMaker.addParam('params', paramsDict, notQuotes=True)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res



import hashlib
import os
from typing import List

from ultipa.structs import Algo

from ultipa.configuration.RequestConfig import RequestConfig
from ultipa.operate.base_extra import BaseExtra
from ultipa.proto import ultipa_pb2
from ultipa.structs.Algo import ALGO_RESULT, HDCAlgo
from ultipa.types import ULTIPA_REQUEST, ULTIPA, ULTIPA_RESPONSE
from ultipa.types.types import Exta
from ultipa.utils import UQLMAKER, CommandList, errors
from ultipa.utils.fileSize import read_in_chunks
from ultipa.utils.format import FormatType
from ultipa.utils.ResposeFormat import ResponseKeyFormat
from ultipa.utils.convert import convertToAlgo, convertToExta


class AlgoExtra(BaseExtra):
	'''
		Processing class that defines settings for algorithm related operations.

	'''
	JSONSTRING_KEYS = ["param"]


	def showExta(self,config: RequestConfig = RequestConfig())-> List[Exta]:
		'''
		Query for algorithm list.

		Args:
			config: An object of the RequestConfig class

		Returns:
			ResponseListAlgo

		'''
		uqlMaker = UQLMAKER(command=CommandList.showExta, commonParams=config)
		res = self.UqlListSimple(uqlMaker)
		if res.status.code == ULTIPA.Code.SUCCESS:
			res.data = convertToExta(res)
			return res.data
		else :
			return res

	def __make_message(self, fileName, md5, chunk, hdcName):
		return ultipa_pb2.InstallAlgoRequest(
			file_name=fileName, md5=md5, chunk=chunk, with_server=ultipa_pb2.WithServer(hdc_server_name=hdcName)
		)

	def __generate_messages(self, request: ULTIPA_REQUEST.InstallAlgo):
		messages = []
		file_object = open(request.soPath, 'rb')
		hdcName = request.hdcName
		somd5 = hashlib.md5(file_object.read()).hexdigest()
		file_object.close()

		for chunk in read_in_chunks(request.soPath):
			fileName = os.path.basename(request.soPath)
			messages.append(self.__make_message(fileName, somd5, chunk, hdcName))

		file_object = open(request.configPath, 'rb')
		configmd5 = hashlib.md5(file_object.read()).hexdigest()
		file_object.close()
		for chunk in read_in_chunks(request.configPath):
			fileName = os.path.basename(request.configPath)
			messages.append(self.__make_message(fileName, configmd5, chunk, hdcName))
		for msg in messages:
			yield msg

	def installAlgo(self, soFilePath: str, infoFilePath: str,
					config: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Install an Ultipa standard algorithm.

		Args:
			soFilePath: The directory of the algorithm package (.so)

			infoFilePath: The directory of the algorithm configuration (.yml)

			config: An object of the RequestConfig class

		Returns:
			Response

		'''
		request = ULTIPA_REQUEST.InstallAlgo(infoFilePath, soFilePath)
		clientInfo = self.getClientInfo(graphSetName=config.graphName, useMaster=config.useMaster,
										isGlobal=True)
		response = ULTIPA_RESPONSE.UltipaResponse()
		try:
			if os.path.exists(request.soPath) and os.path.exists(request.configPath):
				installRet = clientInfo.Controlsclient.InstallAlgo(self.__generate_messages(request),
																   metadata=clientInfo.metadata)
				status = FormatType.status(installRet.status)
				response.status = status
		except Exception as e:
			try:
				message = str(e._state.code) + ' : ' + str(e._state.details)
			except:
				message = str(e)

			response.status = ULTIPA.Status(code=ULTIPA.Code.UNKNOW_ERROR, message=message)

		if self.defaultConfig.responseWithRequestInfo:
			response.req = ULTIPA.ReturnReq(self.graphSetName, "InstallAlgo",
											config.useHost if config.useHost else self.host,
											config.retry,
											False)
		return response

	def uninstallAlgo(self, algoName: str,
					  config: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Uninstall an Ultipa standard algorithm.

		Args:
			algoName: The name of algorithm

			config: An object of the RequestConfig class

		Returns:
			Response

		'''
		clientInfo = self.getClientInfo(graphSetName=config.graphName, useMaster=config.useMaster,
										isGlobal=True)
		arequest = ultipa_pb2.UninstallAlgoRequest(algo_name=algoName)
		installRet = clientInfo.Controlsclient.UninstallAlgo(arequest, metadata=clientInfo.metadata)
		status = FormatType.status(installRet.status)
		response = ULTIPA_RESPONSE.UltipaResponse(status=status)
		if self.defaultConfig.responseWithRequestInfo:
			response.req = ULTIPA.ReturnReq(self.graphSetName, "UnInstallAlgo",
											config.useHost if config.useHost else self.host,
											config.retry,
											False)
		return response


	def installHDCAlgo(self, soFile: str, ymlFile: str,hdcName:str,
					config: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Install an Ultipa standard algorithm.

		Args:
			soFile: The directory of the algorithm package (.so)

			ymlFile: The directory of the algorithm configuration (.yml)

			hdcName: The name of the hdc

			config: An object of the RequestConfig class

		Returns:
			UltipaResponse

		'''
		request = ULTIPA_REQUEST.InstallAlgo(ymlFile, soFile,hdcName)
		clientInfo = self.getClientInfo(graphSetName=config.graphName, useMaster=config.useMaster,
										isGlobal=True)
		response = ULTIPA_RESPONSE.Response()
		try:
			if os.path.exists(request.soPath) and os.path.exists(request.configPath):
				installRet = clientInfo.Controlsclient.InstallAlgo(self.__generate_messages(request),
																   metadata=clientInfo.metadata)
				status = FormatType.status(installRet.status)
				response.status = status
		except Exception as e:
			try:
				message = str(e._state.code) + ' : ' + str(e._state.details)
			except:
				message = str(e)

			response.status = ULTIPA.Status(code=ULTIPA.Code.UNKNOW_ERROR, message=message)

		if self.defaultConfig.responseWithRequestInfo:
			response.req = ULTIPA.ReturnReq(self.graphSetName, "InstallAlgo",
											config.useHost if config.useHost else self.host,
											config.retry,
											False)
		return response

	def uninstallHDCAlgo(self, algoName: str,hdcName:str,
					  config: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Uninstall an Ultipa standard algorithm.

		Args:
			algoName: The name of algorithm

			hdcName: The name of hdc

			config: An object of the RequestConfig class

		Returns:
			Response

		'''
		clientInfo = self.getClientInfo(graphSetName=config.graphName, useMaster=config.useMaster,
										isGlobal=True)
		arequest = ultipa_pb2.UninstallAlgoRequest(algo_name=algoName, with_server=ultipa_pb2.WithServer(hdc_server_name=hdcName))
		installRet = clientInfo.Controlsclient.UninstallAlgo(arequest, metadata=clientInfo.metadata)
		status = FormatType.status(installRet.status)
		response = ULTIPA_RESPONSE.UltipaResponse(status=status)
		if self.defaultConfig.responseWithRequestInfo:
			response.req = ULTIPA.ReturnReq(self.graphSetName, "UnInstallAlgo",
											config.useHost if config.useHost else self.host,
											config.retry,
											False)
		return response

	def rollbackHDCAlgo(self,algoName :str,hdcName:str
						,config: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		"""
		Args:
			algoName: The name of algorithm

			hdcName: The name of hdc

			config: An object of the RequestConfig class

		Returns:
			Response
		"""
		clientInfo = self.getClientInfo(graphSetName=config.graphName, useMaster=config.useMaster,
										isGlobal=True)
		arequest = ultipa_pb2.RollbackAlgoRequest(algo_name=algoName,
												   with_server=ultipa_pb2.WithServer(hdc_server_name=hdcName))
		rollbackRet = clientInfo.Controlsclient.RollbackAlgo(arequest, metadata=clientInfo.metadata)
		status = FormatType.status(rollbackRet.status)
		response = ULTIPA_RESPONSE.UltipaResponse(status=status)
		if self.defaultConfig.responseWithRequestInfo:
			response.req = ULTIPA.ReturnReq(self.graphSetName, "RollbackAlgo",
											config.useHost if config.useHost else self.host,
											config.retry,
											False)
		return response

	def showHDCAlgo(self,hdcName:str
						,config: RequestConfig = RequestConfig()) -> List[HDCAlgo]:
		"""
		Args:
			hdcName: The name of hdc

			config: An object of the RequestConfig class

		Returns:
			Response
		"""
		command=CommandList.showHDCAlgo
		commandp=f"'{hdcName}'"
		uqlMaker = UQLMAKER(command=command, commandP=commandp, commonParams=config)
		res=self.uqlSingle(uqlMaker)
		result = None
		if res.status.code == 0:
			if res.alias(f"_algoList_from_{hdcName}").data:
				result = res.alias(f"_algoList_from_{hdcName}").asHDCAlgos()



		return result



# -*- coding: utf-8 -*-
# @Time    : 2023/8/1 10:47
# @Author  : Ultipa
# @Email   : support@ultipa.com
# @File    : Graph.py
from typing import Dict, List

from ultipa.structs.BaseModel import BaseModel
from ultipa.structs.Path import Path
from ultipa.structs.Node import Node
from ultipa.structs.Edge import Edge
class Graph(BaseModel):
	'''
	    Data class for graphset.
	'''

	# def __init__(self, name: str, description: str = None):
	# 	self.name = name
	# 	self.description = description

	def __init__(self, paths: Path = None,nodes: Dict[int , Node] = None,
				 edges: Dict[int , Edge] = None):
		self.paths = paths
		self.nodes = nodes
		self.edges = edges

class GraphAlias(BaseModel):
	def __init__(self,  alias: str, graph: List[Graph] = None):
		self.alias = alias
		self.graph = graph

	# id: str
	# name: str
	# totalNodes: str
	# totalEdges: str
	# description: str
	# status: str
	#
	# # def __init__(self, name: str, description: str = None):
	# # 	self.name = name
	# # 	self.description = description
	#
	# def __init__(self, name: str, id: int = None, totalNodes: int = None,
	# 			 totalEdges: int = None, description: str = None, status: str = None):
	# 	self.id = id
	# 	self.name = name
	# 	self.totalNodes = totalNodes
	# 	self.totalEdges = totalEdges
	# 	self.description = description
	# 	self.status = status




from typing import List, Tuple, Dict

from ultipa.structs.Schema import SchemaPair
from ultipa.utils.convert import convertTableToDict

from ultipa.configuration.RequestConfig import RequestConfig
from ultipa.operate.base_extra import BaseExtra
from ultipa.structs import Schema
from ultipa.structs import DBType
from ultipa.types import ULTIPA, ULTIPA_REQUEST, ULTIPA_RESPONSE
from ultipa.utils import UQLMAKER, CommandList
from ultipa.utils.ResposeFormat import ResponseKeyFormat
from typing import Tuple, List
from ultipa.operate.property_extra import PropertyExtra
from ultipa.structs.Property import Property

BOOL_KEYS = ["index", "lte"]
REPLACE_KEYS = {
	"name": "schemaName",
	"type": "propertyType",
}


class SchemaExtra(BaseExtra):
	'''
		Prcessing class that defines settings for schema related operations.
	'''

	def createSchema(self, schema: Schema, isCreateProperties: bool = False,
					 requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
        Create a schema.

        Args:
            schema: An object of Schema class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''

		command = schema.DBType == DBType.DBNODE and CommandList.createNodeSchema or CommandList.createEdgeSchema
		commandP = [f"`{schema.name}`"]
		if schema.description:
			commandP.append(schema.description)
		uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
		uqlMaker.setCommandParams(commandP=commandP)
		res = self.uqlSingle(uqlMaker=uqlMaker)

		if isCreateProperties:
			if schema.properties:
				for prop in schema.properties:
					try:
						res1 = PropertyExtra.createProperty(self, dbType=schema.DBType, schemaName=schema.name,
															prop=prop, requestConfig=requestConfig)
					except Exception as e:
						print("An error occurred while creating property:", prop.name, "Error:", e)
		return res


	def showSchema(self,
				   requestConfig: RequestConfig = RequestConfig()) -> Dict:
		# , dbType: DBType = None, schemaName: str = None
		'''
		Show schema(s).

		Args:
			dbType: The DBType of data (DBNODE or DBEDGE), show both types of schemas by default

			schemaName: The name of designated schema, or show all schemas by default

			requestConfig: An object of RequestConfig class

		Returns:
			UltipaResponse

		'''

		# if dbType != None:
		# 	if dbType == DBType.DBNODE:
		# 		command = CommandList.showNodeSchema
		# 	elif dbType == DBType.DBEDGE:
		# 		command = CommandList.showEdgeSchema
		# 	else:
		# 		command = CommandList.showSchema
		#
		# 	if schemaName:
		# 		commandP = '@' + schemaName
		# 	else:
		# 		commandP = ''
		# else:
		command = CommandList.showSchema
		commandP = ''

		uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
		res = self.uqlSingle(uqlMaker)
		graphcount = convertTableToDict(res.alias("_graphCount").data.rows, res.alias("_graphCount").data.headers)
		schemadict = {}
		count={}
		for data in graphcount:
			if data.get('type') in ['total_nodes', 'total_edges']:
				count[data.get('type')]={"count":data.get('count'),'from_schema':data.get('from_schema'),'to_schema':data.get('to_schema')}
			else :
				count[data.get('schema')] = {"count": data.get('count'), 'from_schema': data.get('from_schema'),
										   'to_schema': data.get('to_schema')}
		nodeschemas = res.alias("_nodeSchema").asSchemas()
		edgeschemas = res.alias("_edgeSchema").asSchemas()
		nodeschemas.extend(edgeschemas)
		for data in nodeschemas:
			if data.name in count:
				data.total=count.get(data.name).get('count')
				data.pair = SchemaPair(fromSchema=count.get(data.name).get('from_schema'),toSchema=count.get(data.name).get('to_schema'),count=count.get(data.name).get('count'))

		schemadict['schemas'] = nodeschemas
		schemadict['totalNodes'] = count.get('total_nodes').get('count')
		schemadict['totalEdges'] = count.get('total_edges').get('count')
		return schemadict

	def showNodeSchema(self, requestConfig: RequestConfig = RequestConfig()) -> List[Schema]:
		'''
        Show Nodeschema(s).

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Schema]

        '''

		command = CommandList.showNodeSchema
		commandp = ''
		# 	if schemaName:
		# 		commandP = '@' + schemaName
		# 	else:
		# 		commandP = ''
		# else:
		# 	command = CommandList.showSchema
		# 	commandP = ''
		uqlMaker = UQLMAKER(command=command, commandP=commandp, commonParams=requestConfig)
		res = self.uqlSingle(uqlMaker)
		res.items = res.alias("_nodeSchema").asSchemas()
		return res.items

	def showEdgeSchema(self, requestConfig: RequestConfig = RequestConfig()) -> List[Schema]:
		'''
        Show Edgeschema(s).

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Schema]

        '''

		command = CommandList.showEdgeSchema
		commandp = ''
		# 	if schemaName:
		# 		commandP = '@' + schemaName
		# 	else:
		# 		commandP = ''
		# else:
		# 	command = CommandList.showSchema
		# 	commandP = ''

		# uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
		uqlMaker = UQLMAKER(command=command, commandP=commandp, commonParams=requestConfig)
		res = self.uqlSingle(uqlMaker)
		res.items = res.alias("_edgeSchema").asSchemas()
		return res.items

	def alterSchema(self, schema: Schema, newSchema: Schema,
					requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
        Alter schema.

        Args:
            schema: An object of schema class(to be altered)
            newSchema: An object of schema class

        Returns:
             UltipaResponse

        '''
		command = schema.DBType == DBType.DBNODE and CommandList.alterNodeSchema or CommandList.alterEdgeSchema
		# commandP = '@' + schema.name
		commandP = "@`%s`" % (schema.name)
		update_dict = {}
		if newSchema.name:
			update_dict.setdefault('name', newSchema.name)
		if newSchema.description:
			update_dict.update({'description': newSchema.description})
		uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
		uqlMaker.addParam("set", update_dict)
		res = self.uqlSingle(uqlMaker=uqlMaker)
		return res

	def dropSchema(self, schema: Schema,
                   requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
		'''
		Drop schema.

		Args:
			schema: An object of schema class

		Returns:
			 UltipaResponse

		'''

		command = schema.DBType == DBType.DBNODE and CommandList.dropNodeSchema or CommandList.dropEdgeSchema
		# commandP = '@' + schema.name
		commandP = "@`%s`" % (schema.name)

		uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
		res = self.uqlSingle(uqlMaker=uqlMaker)
		return res

	def getSchema(self, schemaName: str, dbType: DBType,
				  requestConfig: RequestConfig = RequestConfig()) -> Schema:
		'''
        Acquire a designated Schema.

        Args:
            schemaName: The name of Schema

            dbType:The DBType of data (DBNODE or DBEDGE)

            requestConfig: An object of RequestConfig class

        Returns:
            Schema
        '''

		if dbType != None:
			if dbType == DBType.DBNODE:
				command = CommandList.showNodeSchema
			elif dbType == DBType.DBEDGE:
				command = CommandList.showEdgeSchema

			if schemaName:
				commandP = "@`%s`" % (schemaName)

		uqlMaker = UQLMAKER(command, commandP=commandP, commonParams=requestConfig)

		res = self.uqlSingle(uqlMaker=uqlMaker)
		if res.status.code == ULTIPA.Code.SUCCESS:
			if dbType == DBType.DBNODE:
				res.items = res.alias("_nodeSchema").asSchemas()[0]
			else:
				res.items = res.alias("_edgeSchema").asSchemas()[0]
		return res.items

	def getNodeSchema(self, schemaName: str,
					  requestConfig: RequestConfig = RequestConfig()) -> Schema:
		'''
        Acquire a designated Node Schema.

        Args:
            schemaName: The name of Schema
            requestConfig: An object of RequestConfig class

        Returns:
            Schema
        '''

		command = CommandList.showNodeSchema

		if schemaName:
			commandP = "@`%s`" % (schemaName)

		uqlMaker = UQLMAKER(command, commandP=commandP, commonParams=requestConfig)

		res = self.uqlSingle(uqlMaker=uqlMaker)
		if res.status.code == ULTIPA.Code.SUCCESS:
			res.items = res.alias("_nodeSchema").asSchemas()[0]
		else:
			res.items = None
		return res.items

	def getEdgeSchema(self, schemaName: str,
					  requestConfig: RequestConfig = RequestConfig()) -> Schema:
		'''
        Acquire a designated Edge Schema.

        Args:
            schemaName: The name of Schema
            requestConfig: An object of RequestConfig class

        Returns:
            ResponseSchema
        '''

		command = CommandList.showEdgeSchema

		if schemaName:
			commandP = "@`%s`" % (schemaName)

		uqlMaker = UQLMAKER(command, commandP=commandP, commonParams=requestConfig)

		res = self.uqlSingle(uqlMaker=uqlMaker)
		if res.status.code == ULTIPA.Code.SUCCESS:
			res.items = res.alias("_edgeSchema").asSchemas()[0]
		else:
			res.items = None
		return res.items

	def createSchemaIfNotExist(self, schema: Schema, isCreateProperties: bool = False,
							   requestConfig: RequestConfig = RequestConfig()) -> Tuple[
		bool, ULTIPA_RESPONSE.UltipaResponse]:
		'''
        Create a schema if schema does not exist.

        Args:
            schema: An object of Schema class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''
		check = self.getSchema(schemaName=schema.name, dbType=schema.DBType, requestConfig=requestConfig)
		if check is not None:
			return [True, None]

		elif check is None:
			res = self.createSchema(schema=schema, isCreateProperties=isCreateProperties, requestConfig=requestConfig)
			return [False, res]

	def getProperty(self, dbType: DBType, propertyName: str, schemaName: str,
					requestConfig: RequestConfig = RequestConfig()) -> Property:
		'''
		Get a property.

		Args:
			dbType: The DBType of data (DBNODE or DBEDGE)

			schemaName: The name of schema

			propertyName: The name of the Property

			requestConfig: An object of RequestConfig class

		Returns:
			Property

		'''
		res = self.getSchema(schemaName=schemaName, dbType=dbType, requestConfig=requestConfig)
		if res :
			properties = res.properties
			for property in properties:
				if property.name == propertyName:
					return property
		return None


	def getNodeProperty(self, propertyName: str, schemaName: str,
						requestConfig: RequestConfig = RequestConfig()) -> Property:
		'''
		Get a Node property.

		Args:
			schemaName: The name of schema

			propertyName: The name of the Property

			requestConfig: An object of RequestConfig class

		Returns:
			Property

		'''
		res = self.getProperty(propertyName=propertyName, schemaName=schemaName, dbType=DBType.DBNODE,
							   requestConfig=requestConfig)
		return res


	def getEdgeProperty(self, propertyName: str, schemaName: str,
						requestConfig: RequestConfig = RequestConfig()) -> Property:
		'''
		Get an Edge property.

		Args:
			schemaName: The name of schema

			propertyName: The name of the Property

			requestConfig: An object of RequestConfig class

		Returns:
			Property

		'''
		res = self.getProperty(propertyName=propertyName, schemaName=schemaName, dbType=DBType.DBEDGE,
							   requestConfig=requestConfig)
		return res


	def createPropertyIfNotExist(self, dbType: DBType, schemaName: str, prop: Property,
								 requestConfig: RequestConfig = RequestConfig()) -> Tuple[
		bool, ULTIPA_RESPONSE.ResponseProperty]:
		'''
		Create a schema if schema does not exist.

		Args:
			schema: An object of Schema class

			requestConfig: An object of RequestConfig class

		Returns:
			Bool
			ResponseGetProperty

		'''

		res = self.getProperty(dbType=dbType, propertyName=prop.name, schemaName=schemaName,
							   requestConfig=requestConfig)
		if res != None:
			return [True, None]
		if res == None:
			newprop = PropertyExtra.createProperty(self, dbType=dbType, schemaName=schemaName, prop=prop,
												   requestConfig=requestConfig)
			return [False, newprop]


# -*- coding: utf-8 -*-
# @Time    : 2024/05/14 11:47
# @Author  : Ultipa
# @Email   : support@ultipa.com
# @File    : Index.py
from typing import List

from ultipa.structs import DBType
from ultipa.structs.BaseModel import BaseModel


class Job(BaseModel):
    '''
        Data class for Index.
    '''

    def __init__(self, id: str, graph: str ='',
                 type: str='',
                 uql: str='',
                 gql: str='',
                 status: str='',
                 error: str='',
                 result: str='',
                 startTime: str='',
                 endTime: str='',
                 progress: str=''):
        self.id = id
        self.graph = graph
        self.type = type
        self.uql = uql
        self.gql = gql
        self.status = status
        self.error = error
        self.result = result
        self.startTime = startTime
        self.endTime = endTime
        self.progress = progress




from typing import List

from ultipa import DBType
from ultipa.operate.base_extra import BaseExtra
from ultipa.structs.Index import Index
from ultipa.structs.Job import Job
from ultipa.types import ULTIPA_REQUEST, ULTIPA, ULTIPA_RESPONSE
from ultipa.structs.Property import Property
from ultipa.structs.Index import Index
from ultipa.utils import UQLMAKER, CommandList
from ultipa.configuration.RequestConfig import RequestConfig
from ultipa.structs.DBType import DBType
from ultipa.utils.convert import convertTableToDict, convertToIndex, convertToFullText


class IndexExtra(BaseExtra):
    '''
	Processing class that defines settings for index related operations.
	'''

    def showIndex(self,
                  requestConfig: RequestConfig = RequestConfig()) -> List[Index]:
        '''
        Show all indice.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Index]
        '''
        # if dbType != None:
        # 	command = dbType == DBType.DBNODE and CommandList.showNodeIndex or CommandList.showEdgeIndex
        # else:
        command = CommandList.showIndex
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
        indexdata = convertToIndex(res=res, all=True)
        if len(indexdata) > 0:
            res.data = indexdata
        else:
            res.data = None
        return res.data

    def showNodeIndex(self, requestConfig: RequestConfig = RequestConfig()) -> List[Index]:
        '''
        Show all Node indice.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Index]
        '''
        # res=self.showIndex(dbType=DBType.DBNODE,requestConfig=requestConfig)
        command = CommandList.showNodeIndex
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
        indexdata = convertToIndex(res=res, all=False, dbtype=DBType.DBNODE)
        if len(indexdata) > 0:
            res.data = indexdata
        else:
            res.data = None
        return res.data

    def showEdgeIndex(self, requestConfig: RequestConfig = RequestConfig()) -> List[Index]:
        '''
        Show all Edge indice.

        Args:
            requestConfig: An object of RequestConfig class

        Returns:
            List[Index]
        '''
        # res=self.showIndex(dbType=DBType.DBEDGE,requestConfig=requestConfig)
        command = CommandList.showEdgeIndex
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
        indexdata = convertToIndex(res=res, all=False, dbtype=DBType.DBEDGE)
        if len(indexdata) > 0:
            res.data = indexdata
        else:
            res.data = None
        return res.data

    def showFulltext(self,
                     requestConfig: RequestConfig = RequestConfig()) -> List[Index]:
        '''
        Show all full-text indice.

        Args:
            requestConfig: An object of RequestConfig class
        Returns:

            List[Index]
        '''
        # if dbType != None:
        # 	command = dbType == DBType.DBNODE and CommandList.showNodeFulltext or CommandList.showEdgeFulltext
        # else:
        command = CommandList.showFulltext
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
        fulltextdata = convertToFullText(res=res)
        if len(fulltextdata) > 0:
            res.data = fulltextdata
        else:
            res.data = None
        return res.data

    # def createIndex(self, dbType: DBType, propertyName: str, schemaName: str = None,
    #                 requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
    #     '''
    #     Create an index.
    #
    #     Args:
    #         dbType: The DBType of data (DBNODE or DBEDGE)
    #
    #         schemaName: The name of schema
    #
    #         property: An object of Property class
    #
    #         requestConfig: An object of RequestConfig class
    #
    #     Returns:
    #         UltipaResponse
    #     '''
    #
    #     command = dbType == DBType.DBNODE and CommandList.createNodeIndex or CommandList.createEdgeIndex
    #     # commandP = request.toString
    #     if schemaName:
    #         commandP = "@`%s`.`%s`" % (schemaName, propertyName)
    #     elif schemaName == None:
    #         commandP = "@`*`.`%s`" % (propertyName)
    #     uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
    #     res = self.uqlSingle(uqlMaker=uqlMaker)
    #     return res

    def createNodeIndex(self, source: str, indexName: str,
                        requestConfig: RequestConfig = RequestConfig()) -> Job:
        '''
		Create an index.

		Args:
			request: An object of CreateIndex class

			requestConfig: An object of RequestConfig class

		Returns:
			UltipaResponse
		'''

        command = CommandList.createNodeIndex

        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        source = f'{source}'
        uqlMaker.setCommandParams([source, indexName])
        res = self.uqlSingle(uqlMaker=uqlMaker)
        if res.status.code ==0:
            result = convertTableToDict(res.alias('result').data.rows, res.alias('result').data.headers)
            return Job(id=result[0].get('new_job_id'))
        else:
            return res

    def createEdgeIndex(self, source: str, indexName: str,
                        requestConfig: RequestConfig = RequestConfig()) -> Job:
        '''
		Create an index.

		Args:
			request: An object of CreateIndex class

			requestConfig: An object of RequestConfig class

		Returns:
			UltipaResponse
		'''

        command = CommandList.createEdgeIndex
        source = f'{source}'
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        uqlMaker.setCommandParams([source, indexName])
        res = self.uqlSingle(uqlMaker=uqlMaker)
        if res.status.code == 0:
            result = convertTableToDict(res.alias('result').data.rows, res.alias('result').data.headers)
            return Job(id=result[0].get('new_job_id'))
        else :
             return res

    def createFulltext(self, dbType: DBType, schemaName: str, propertyName: str, fulltextName: str,
                       requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Create a full-text index.

        Args:
            dbType: The DBType of data (DBNODE or DBEDGE)

            schemaName: The name of schema

            property: An object of Property class

            fulltextName: Name of the fulltext index

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''
        command = dbType == DBType.DBNODE and CommandList.createNodeFulltext or CommandList.createEdgeFulltext
        command1 = "@`%s`.`%s`" % (schemaName, propertyName)
        commandP = [command1, fulltextName]
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        uqlMaker.setCommandParams(commandP=commandP)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res


    def createNodeFulltext(self,schemaName: str, propertyName: str, fulltextName: str,
                           rquestConfig: RequestConfig = RequestConfig()) -> List[Job]:
        '''
        Create a full-text index.

        Args:
            request: An object of CreateFulltext class

            rquestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse

        '''
        command = CommandList.createNodeFulltext
        command1 = "@`%s`.`%s`" % (schemaName, propertyName)
        commandP=[command1, fulltextName]
        uqlMaker = UQLMAKER(command=command, commonParams=rquestConfig)
        uqlMaker.setCommandParams(commandP=commandP)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        if res.status.code == 0:
            result = convertTableToDict(res.alias('result').data.rows, res.alias('result').data.headers)
            return [Job(id=data.get('new_job_id')) for data in result]
        else:
            return res

    def createEdgeFulltext(self,schemaName: str, propertyName: str, fulltextName: str,
                           rquestConfig: RequestConfig = RequestConfig()) -> List[Job]:
        '''
		Create a full-text index.

		Args:
			request: An object of CreateFulltext class

			rquestConfig: An object of RequestConfig class

		Returns:
			UltipaResponse

		'''
        command = CommandList.createEdgeFulltext
        command1 = "@`%s`.`%s`" % (schemaName, propertyName)
        commandP = [command1, fulltextName]
        uqlMaker = UQLMAKER(command=command, commonParams=rquestConfig)
        uqlMaker.setCommandParams(commandP=commandP)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        if res.status.code == 0:
            result = convertTableToDict(res.alias('result').data.rows, res.alias('result').data.headers)
            return [Job(id=data.get('new_job_id')) for data in result]
        else:
            return res

    def dropIndex(self, dbType:DBType,indexName:str,
                  requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Drop an index.

        Args:
            request: An object of DropIndex class

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse
        '''
        command = dbType == DBType.DBNODE and CommandList.dropNodeIndex or CommandList.dropEdgeIndex
        commandP = f'"{indexName}"'

        uqlMaker = UQLMAKER(command=command, commandP=commandP, commonParams=requestConfig)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res

    def dropFulltext(self, dbType: DBType, fulltextName: str,
                     requestConfig: RequestConfig = RequestConfig()) -> ULTIPA_RESPONSE.UltipaResponse:
        '''
        Drop a full-text index.

        Args:
            dbType: The DBType of data (DBNODE or DBEDGE)

            fulltextName: Name of the fulltext index

            requestConfig: An object of RequestConfig class

        Returns:
            UltipaResponse
        '''

        command = dbType == DBType.DBNODE and CommandList.dropNodeFulltext or CommandList.dropEdgeFulltext
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        uqlMaker.setCommandParams(fulltextName)
        res = self.uqlSingle(uqlMaker=uqlMaker)
        return res

    def showNodeFulltext(self, requestConfig: RequestConfig = RequestConfig()) -> List[Index]:
        '''
        Show all full-text Node indice.

        Args:

            requestConfig: An object of RequestConfig class

        Returns:

            List[Index]
        '''
        # res=self.showFulltext(dbType=DBType.DBNODE,requestConfig=requestConfig)
        command = CommandList.showNodeFulltext
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
        fulltextdata = convertToFullText(res=res, all=False, dbtype=DBType.DBNODE)
        if len(fulltextdata) > 0:
            res.data = fulltextdata
        else:
            res.data = None
        return res.data

    def showEdgeFulltext(self, requestConfig: RequestConfig = RequestConfig()) -> List[Index]:
        '''
        Show all full-text Edge indice.

        Args:

            requestConfig: An object of RequestConfig class

        Returns:

            List[Index]
        '''
        # res=self.showFulltext(dbType=DBType.DBEDGE,requestConfig=requestConfig)
        command = CommandList.showEdgeFulltext
        uqlMaker = UQLMAKER(command=command, commonParams=requestConfig)
        res = self.UqlListSimple(uqlMaker=uqlMaker, isSingleOne=False)
        fulltextdata = convertToFullText(res=res, all=False, dbtype=DBType.DBEDGE)
        if len(fulltextdata) > 0:
            res.data = fulltextdata
        else:
            res.data = None
        return res.data



# -*- coding: utf-8 -*-
# @Time    : 2023/8/4 16:19
# @Author  : Ultipa
# @Email   : support@ultipa.com
# @File    : Edge.py
from typing import Dict

from ultipa.structs.BaseModel import BaseModel


class Edge(BaseModel):
	'''
	    Data class for edge.
	'''
	_index = None
	def getUUID(self):
		return self._uuid



	def __init__(self,
				 _uuid: int = None,
				 _from: str =  '',
				 _to: str = '',
				 _from_uuid: str = '',
				 _to_uuid:str='',
				 schema: str = None,
				 values: Dict = None):

		self.schema = schema
		self._uuid = _uuid
		self._from = _from
		self._from_uuid=_from_uuid
		self._to = _to
		self._to_uuid = _to_uuid
		self.values = values

	def getFrom(self):
		return self._from

	def getTo(self):
		return self._to

	def getId(self):
		return self._id

	def getFromUUID(self):
		return self._from_uuid

	def getToUUID(self):
		return self._to_uuid

	def getValues(self):
		return self.values

	def getSchema(self):
		return self.schema

	def get(self, propertyName: str):
		return self.values.get(propertyName)

	def set(self, propertyName: str, value):
		self.values.update({propertyName: value})

	def _getIndex(self):
		return self._index


from ultipa.structs.BaseModel import BaseModel

class Projection(BaseModel):

    def __init__(self,name: str,type: str,filterType: str, isDefault: str,sourceGraph: str,status: str,stats: str,hdcName: str,hdcStatus: str,config: str):
        self.name = name
        self.type = type
        self.filterType = filterType
        self.isDefault = isDefault
        self.souceGraph = sourceGraph
        self.status = status
        self.stats = stats
        self.hdcName = hdcName
        self.hdcStatus = hdcStatus
        self.config = config


# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ultipa.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0cultipa.proto\x12\x06ultipa\"\"\n\x12HelloUltipaRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\"C\n\x10HelloUltipaReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\x0f\n\x07message\x18\x02 \x01(\t\"\xb4\x01\n\x0cQueryRequest\x12\x12\n\nquery_text\x18\x01 \x01(\t\x12%\n\nquery_type\x18\x02 \x01(\x0e\x32\x11.ultipa.QueryType\x12\x0f\n\x07timeout\x18\x03 \x01(\r\x12\x12\n\ngraph_name\x18\x04 \x01(\t\x12\x11\n\tuser_name\x18\x05 \x01(\t\x12\x12\n\nthread_num\x18\x06 \x01(\r\x12\n\n\x02tz\x18\x07 \x01(\t\x12\x11\n\ttz_offset\x18\x08 \x01(\t\"E\n\x0bResultAlias\x12\r\n\x05\x61lias\x18\x01 \x01(\t\x12\'\n\x0bresult_type\x18\x02 \x01(\x0e\x32\x12.ultipa.ResultType\"\xf8\x02\n\nQueryReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\x17\n\x0ftotal_time_cost\x18\x02 \x01(\x05\x12\x18\n\x10\x65ngine_time_cost\x18\x03 \x01(\x05\x12\"\n\x05\x61lias\x18\x04 \x03(\x0b\x32\x13.ultipa.ResultAlias\x12 \n\x05paths\x18\x05 \x03(\x0b\x32\x11.ultipa.PathAlias\x12 \n\x05nodes\x18\x06 \x03(\x0b\x32\x11.ultipa.NodeAlias\x12 \n\x05\x65\x64ges\x18\x07 \x03(\x0b\x32\x11.ultipa.EdgeAlias\x12 \n\x05\x61ttrs\x18\x08 \x03(\x0b\x32\x11.ultipa.AttrAlias\x12\x1d\n\x06tables\x18\n \x03(\x0b\x32\r.ultipa.Table\x12!\n\nstatistics\x18\x0b \x01(\x0b\x32\r.ultipa.Table\x12)\n\x0c\x65xplain_plan\x18\x0c \x01(\x0b\x32\x13.ultipa.ExplainPlan\"7\n\tPathAlias\x12\x1b\n\x05paths\x18\x01 \x03(\x0b\x32\x0c.ultipa.Path\x12\r\n\x05\x61lias\x18\x02 \x01(\t\"C\n\tEdgeAlias\x12\'\n\nedge_table\x18\x01 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\r\n\x05\x61lias\x18\x02 \x01(\t\"C\n\tNodeAlias\x12\'\n\nnode_table\x18\x01 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\r\n\x05\x61lias\x18\x02 \x01(\t\"6\n\tAttrAlias\x12\r\n\x05\x61lias\x18\x01 \x01(\t\x12\x1a\n\x04\x61ttr\x18\x02 \x01(\x0b\x32\x0c.ultipa.Attr\"@\n\x04\x41ttr\x12(\n\nvalue_type\x18\x01 \x01(\x0e\x32\x14.ultipa.PropertyType\x12\x0e\n\x06values\x18\x02 \x03(\x0c\"\xc3\x01\n\x0c\x41ttrListData\x12 \n\x04type\x18\x01 \x01(\x0e\x32\x12.ultipa.ResultType\x12\"\n\x05nodes\x18\x02 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\"\n\x05\x65\x64ges\x18\x03 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\x1b\n\x05paths\x18\x04 \x03(\x0b\x32\x0c.ultipa.Path\x12\x1b\n\x05\x61ttrs\x18\x05 \x03(\x0b\x32\x0c.ultipa.Attr\x12\x0f\n\x07is_null\x18\x06 \x01(\x08\"V\n\x0b\x41ttrMapData\x12\x19\n\x03key\x18\x01 \x01(\x0b\x32\x0c.ultipa.Attr\x12\x1b\n\x05value\x18\x02 \x01(\x0b\x32\x0c.ultipa.Attr\x12\x0f\n\x07is_null\x18\x03 \x01(\x08\"\x97\x01\n\x12UserSettingRequest\x12\x11\n\tuser_name\x18\x01 \x01(\t\x12.\n\x03opt\x18\x02 \x01(\x0e\x32!.ultipa.UserSettingRequest.OPTION\x12\x0c\n\x04type\x18\x03 \x01(\t\x12\x0c\n\x04\x64\x61ta\x18\x04 \x01(\t\"\"\n\x06OPTION\x12\x0b\n\x07OPT_GET\x10\x00\x12\x0b\n\x07OPT_SET\x10\x01\"@\n\x10UserSettingReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\t\"(\n\x13\x44ownloadFileRequest\x12\x11\n\tfile_name\x18\x01 \x01(\t\"V\n\x11\x44ownloadFileReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\x12\n\ntotal_size\x18\x02 \x01(\x05\x12\r\n\x05\x63hunk\x18\x03 \x01(\x0c\"y\n\rExportRequest\x12\x1f\n\x07\x64\x62_type\x18\x01 \x01(\x0e\x32\x0e.ultipa.DBType\x12\r\n\x05limit\x18\x02 \x01(\x05\x12\x19\n\x11select_properties\x18\x03 \x03(\t\x12\x0e\n\x06schema\x18\x04 \x01(\t\x12\r\n\x05graph\x18\x05 \x01(\t\"\x7f\n\x0b\x45xportReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\'\n\nnode_table\x18\x02 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\'\n\nedge_table\x18\x03 \x01(\x0b\x32\x13.ultipa.EntityTable\"\xc2\x01\n\x12InsertNodesRequest\x12\'\n\nnode_table\x18\x01 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\x0e\n\x06silent\x18\x02 \x01(\x08\x12\x12\n\ngraph_name\x18\x03 \x01(\t\x12\'\n\x0binsert_type\x18\x04 \x01(\x0e\x32\x12.ultipa.InsertType\x12\x10\n\x08internal\x18\x05 \x01(\x08\x12\x13\n\x0bprojections\x18\x06 \x03(\t\x12\x0f\n\x07indexes\x18\x07 \x03(\r\"\xd6\x01\n\x10InsertNodesReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\x11\n\ttime_cost\x18\x02 \x01(\x05\x12\x18\n\x10\x65ngine_time_cost\x18\x03 \x01(\x05\x12\x11\n\x05uuids\x18\x04 \x03(\x04\x42\x02\x30\x01\x12\x0b\n\x03ids\x18\x05 \x03(\t\x12\x16\n\x0eignore_indexes\x18\x06 \x03(\r\x12\x19\n\x11ignore_error_code\x18\x07 \x03(\r\x12\"\n\x05nodes\x18\x08 \x01(\x0b\x32\x13.ultipa.EntityTable\"\xd2\x01\n\x12InsertEdgesRequest\x12\'\n\nedge_table\x18\x01 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\x0e\n\x06silent\x18\x02 \x01(\x08\x12\x12\n\ngraph_name\x18\x03 \x01(\t\x12 \n\x18\x63reate_node_if_not_exist\x18\x04 \x01(\x08\x12\'\n\x0binsert_type\x18\x05 \x01(\x0e\x32\x12.ultipa.InsertType\x12\x13\n\x0bprojections\x18\x06 \x03(\t\x12\x0f\n\x07indexes\x18\x07 \x03(\r\"\xc9\x01\n\x10InsertEdgesReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\x12\x11\n\ttime_cost\x18\x02 \x01(\x05\x12\x18\n\x10\x65ngine_time_cost\x18\x03 \x01(\x05\x12\x11\n\x05uuids\x18\x04 \x03(\x04\x42\x02\x30\x01\x12\x16\n\x0eignore_indexes\x18\x05 \x03(\r\x12\x19\n\x11ignore_error_code\x18\x06 \x03(\r\x12\"\n\x05\x65\x64ges\x18\x07 \x01(\x0b\x32\x13.ultipa.EntityTable\"%\n\nWithServer\x12\x17\n\x0fhdc_server_name\x18\x01 \x01(\t\"l\n\x12InstallAlgoRequest\x12\x11\n\tfile_name\x18\x01 \x01(\t\x12\x0b\n\x03md5\x18\x02 \x01(\t\x12\r\n\x05\x63hunk\x18\x03 \x01(\x0c\x12\'\n\x0bwith_server\x18\x04 \x01(\x0b\x32\x12.ultipa.WithServer\"2\n\x10InstallAlgoReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\"R\n\x14UninstallAlgoRequest\x12\x11\n\talgo_name\x18\x01 \x01(\t\x12\'\n\x0bwith_server\x18\x02 \x01(\x0b\x32\x12.ultipa.WithServer\"4\n\x12UninstallAlgoReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\"Q\n\x13RollbackAlgoRequest\x12\x11\n\talgo_name\x18\x01 \x01(\t\x12\'\n\x0bwith_server\x18\x02 \x01(\x0b\x32\x12.ultipa.WithServer\"3\n\x11RollbackAlgoReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\"g\n\x06Status\x12%\n\nerror_code\x18\x01 \x01(\x0e\x32\x11.ultipa.ErrorCode\x12\x0b\n\x03msg\x18\x02 \x01(\t\x12)\n\x0c\x63luster_info\x18\x03 \x01(\x0b\x32\x13.ultipa.ClusterInfo\"k\n\x0cRaftFollower\x12\x0f\n\x07\x61\x64\x64ress\x18\x01 \x01(\t\x12\x0c\n\x04role\x18\x02 \x01(\x05\x12$\n\x06status\x18\x03 \x01(\x0e\x32\x14.ultipa.ServerStatus\x12\x16\n\x0estream_address\x18\x04 \x01(\t\"\x7f\n\x0b\x43lusterInfo\x12\x10\n\x08redirect\x18\x01 \x01(\t\x12\x16\n\x0eleader_address\x18\x02 \x01(\t\x12\'\n\tfollowers\x18\x03 \x03(\x0b\x32\x14.ultipa.RaftFollower\x12\x1d\n\x15leader_stream_address\x18\x04 \x01(\t\"i\n\x04Path\x12\'\n\nnode_table\x18\x01 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\'\n\nedge_table\x18\x02 \x01(\x0b\x32\x13.ultipa.EntityTable\x12\x0f\n\x07is_null\x18\x03 \x01(\x08\"b\n\x05Table\x12\x12\n\ntable_name\x18\x01 \x01(\t\x12\x1f\n\x07headers\x18\x02 \x03(\x0b\x32\x0e.ultipa.Header\x12$\n\ntable_rows\x18\x03 \x03(\x0b\x32\x10.ultipa.TableRow\"\x1a\n\x08TableRow\x12\x0e\n\x06values\x18\x01 \x03(\x0c\"V\n\x0b\x45ntityTable\x12\x1f\n\x07schemas\x18\x01 \x03(\x0b\x32\x0e.ultipa.Schema\x12&\n\x0b\x65ntity_rows\x18\x03 \x03(\x0b\x32\x11.ultipa.EntityRow\"\xec\x01\n\tEntityRow\x12\x10\n\x04uuid\x18\x01 \x01(\x04\x42\x02\x30\x01\x12\n\n\x02id\x18\x02 \x01(\t\x12\x13\n\x0bschema_name\x18\x03 \x01(\t\x12\x15\n\tfrom_uuid\x18\x04 \x01(\x04\x42\x02\x30\x01\x12\x13\n\x07to_uuid\x18\x05 \x01(\x04\x42\x02\x30\x01\x12\x0f\n\x07\x66rom_id\x18\x06 \x01(\t\x12\r\n\x05to_id\x18\x07 \x01(\t\x12\x0e\n\x06values\x18\x08 \x03(\x0c\x12\x0f\n\x07is_null\x18\t \x01(\x08\x12\x11\n\tschema_id\x18\n \x01(\r\x12\x16\n\x0e\x66rom_schema_id\x18\x0b \x01(\r\x12\x14\n\x0cto_schema_id\x18\x0c \x01(\r\"V\n\x06Schema\x12\x13\n\x0bschema_name\x18\x01 \x01(\t\x12$\n\nproperties\x18\x02 \x03(\x0b\x32\x10.ultipa.Property\x12\x11\n\tschema_id\x18\x03 \x01(\r\"\x8c\x01\n\x08Property\x12\x15\n\rproperty_name\x18\x01 \x01(\t\x12+\n\rproperty_type\x18\x02 \x01(\x0e\x32\x14.ultipa.PropertyType\x12\'\n\tsub_types\x18\x03 \x03(\x0e\x32\x14.ultipa.PropertyType\x12\x13\n\x0bproperty_id\x18\x04 \x01(\r\"L\n\x06Header\x12\x15\n\rproperty_name\x18\x01 \x01(\t\x12+\n\rproperty_type\x18\x02 \x01(\x0e\x32\x14.ultipa.PropertyType\"#\n\x05Value\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"+\n\x08ListData\x12\x0e\n\x06values\x18\x01 \x03(\x0c\x12\x0f\n\x07is_null\x18\x02 \x01(\x08\"*\n\x07SetData\x12\x0e\n\x06values\x18\x01 \x03(\x0c\x12\x0f\n\x07is_null\x18\x02 \x01(\x08\"&\n\x08MapValue\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x0c\"<\n\x07MapData\x12 \n\x06values\x18\x01 \x03(\x0b\x32\x10.ultipa.MapValue\x12\x0f\n\x07is_null\x18\x02 \x01(\x08\"3\n\x0b\x45xplainPlan\x12$\n\nplan_nodes\x18\x01 \x03(\x0b\x32\x10.ultipa.PlanNode\"R\n\x08PlanNode\x12\r\n\x05\x61lias\x18\x01 \x01(\t\x12\x14\n\x0c\x63hildren_num\x18\x02 \x01(\r\x12\x12\n\nquery_text\x18\x03 \x01(\t\x12\r\n\x05infos\x18\x04 \x01(\t\"Q\n\x13\x41uthenticateRequest\x12&\n\x04type\x18\x01 \x01(\x0e\x32\x18.ultipa.AuthenticateType\x12\x12\n\nquery_text\x18\x02 \x01(\t\"3\n\x11\x41uthenticateReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status\"+\n\rMigrationData\x12\x0b\n\x03key\x18\x01 \x01(\x0c\x12\r\n\x05value\x18\x02 \x01(\x0c\":\n\x12MigrationDataArray\x12$\n\x05\x64\x61tas\x18\x01 \x03(\x0b\x32\x15.ultipa.MigrationData\"&\n\x10GetLeaderRequest\x12\x12\n\ngraph_name\x18\x01 \x01(\t\"0\n\x0eGetLeaderReply\x12\x1e\n\x06status\x18\x01 \x01(\x0b\x32\x0e.ultipa.Status*.\n\tQueryType\x12\x0f\n\x0bQUERY_UNSET\x10\x00\x12\x07\n\x03UQL\x10\x01\x12\x07\n\x03GQL\x10\x02*\x92\x01\n\nResultType\x12\x15\n\x11RESULT_TYPE_UNSET\x10\x00\x12\x14\n\x10RESULT_TYPE_PATH\x10\x01\x12\x14\n\x10RESULT_TYPE_NODE\x10\x02\x12\x14\n\x10RESULT_TYPE_EDGE\x10\x03\x12\x14\n\x10RESULT_TYPE_ATTR\x10\x04\x12\x15\n\x11RESULT_TYPE_TABLE\x10\x05*3\n\nInsertType\x12\n\n\x06NORMAL\x10\x00\x12\r\n\tOVERWRITE\x10\x01\x12\n\n\x06UPSERT\x10\x02*\xe4\x01\n\x0cPropertyType\x12\t\n\x05UNSET\x10\x00\x12\t\n\x05INT32\x10\x01\x12\n\n\x06UINT32\x10\x02\x12\t\n\x05INT64\x10\x03\x12\n\n\x06UINT64\x10\x04\x12\t\n\x05\x46LOAT\x10\x05\x12\n\n\x06\x44OUBLE\x10\x06\x12\n\n\x06STRING\x10\x07\x12\x0c\n\x08\x44\x41TETIME\x10\x08\x12\r\n\tTIMESTAMP\x10\t\x12\x08\n\x04TEXT\x10\n\x12\x08\n\x04\x42LOB\x10\x0b\x12\t\n\x05POINT\x10\x0c\x12\x0b\n\x07\x44\x45\x43IMAL\x10\r\x12\x08\n\x04LIST\x10\x0e\x12\x07\n\x03SET\x10\x0f\x12\x07\n\x03MAP\x10\x10\x12\t\n\x05NULL_\x10\x11\x12\x08\n\x04\x42OOL\x10\x12*\xcd\x02\n\tErrorCode\x12\x0b\n\x07SUCCESS\x10\x00\x12\n\n\x06\x46\x41ILED\x10\x01\x12\x0f\n\x0bPARAM_ERROR\x10\x02\x12\x11\n\rBASE_DB_ERROR\x10\x03\x12\x10\n\x0c\x45NGINE_ERROR\x10\x04\x12\x10\n\x0cSYSTEM_ERROR\x10\x05\x12\x10\n\x0cSYNTAX_ERROR\x10\x06\x12\x15\n\x11PERMISSION_DENIED\x10\x07\x12\x10\n\x0c\x44UPLICATE_ID\x10\x08\x12\x12\n\rRAFT_REDIRECT\x10\x91N\x12 \n\x1bRAFT_LEADER_NOT_YET_ELECTED\x10\x92N\x12\x13\n\x0eRAFT_LOG_ERROR\x10\x93N\x12\x12\n\rNOT_RAFT_MODE\x10\x94N\x12 \n\x1bRAFT_NO_AVAILABLE_FOLLOWERS\x10\x95N\x12#\n\x1eRAFT_NO_AVAILABLE_ALGO_SERVERS\x10\x96N*]\n\x0c\x46ollowerRole\x12\x0e\n\nROLE_UNSET\x10\x00\x12\x11\n\rROLE_READABLE\x10\x01\x12\x18\n\x14ROLE_ALGO_EXECUTABLE\x10\x02\x12\x10\n\x0cROLE_LEARNER\x10\x04*<\n\x0cServerStatus\x12\x17\n\x13SERVER_STATUS_UNSET\x10\x00\x12\t\n\x05\x41LIVE\x10\x01\x12\x08\n\x04\x44\x45\x41\x44\x10\x02*.\n\x06\x44\x42Type\x12\n\n\x06\x44\x42NODE\x10\x00\x12\n\n\x06\x44\x42\x45\x44GE\x10\x01\x12\x0c\n\x08\x44\x42GLOBAL\x10\x02*p\n\nJOB_STATUS\x12\x0f\n\x0bJOB_PENDING\x10\x00\x12\x11\n\rJOB_COMPUTING\x10\x01\x12\x0f\n\x0bJOB_WRITING\x10\x02\x12\x0c\n\x08JOB_DONE\x10\x03\x12\x0e\n\nJOB_FAILED\x10\x04\x12\x0f\n\x0bJOB_STOPPED\x10\x05*\xc5\x02\n\x10\x41uthenticateType\x12\x19\n\x15PERMISSION_TYPE_UNSET\x10\x00\x12\x17\n\x13PERMISSION_TYPE_UQL\x10\x01\x12\x17\n\x13PERMISSION_TYPE_GQL\x10\x02\x12\x1f\n\x1bPERMISSION_TYPE_INSERTNODES\x10\x03\x12\x1f\n\x1bPERMISSION_TYPE_INSERTEDGES\x10\x04\x12\x1a\n\x16PERMISSION_TYPE_EXPORT\x10\x05\x12 \n\x1cPERMISSION_TYPE_DOWNLOADFILE\x10\x06\x12\x1f\n\x1bPERMISSION_TYPE_INSTALLALGO\x10\x07\x12!\n\x1dPERMISSION_TYPE_UNINSTALLALGO\x10\x08\x12 \n\x1cPERMISSION_TYPE_ROLLBACKALGO\x10\t2\x95\x02\n\nUltipaRpcs\x12\x42\n\x08SayHello\x12\x1a.ultipa.HelloUltipaRequest\x1a\x18.ultipa.HelloUltipaReply\"\x00\x12\x35\n\x05Query\x12\x14.ultipa.QueryRequest\x1a\x12.ultipa.QueryReply\"\x00\x30\x01\x12\x45\n\x0bInsertNodes\x12\x1a.ultipa.InsertNodesRequest\x1a\x18.ultipa.InsertNodesReply\"\x00\x12\x45\n\x0bInsertEdges\x12\x1a.ultipa.InsertEdgesRequest\x1a\x18.ultipa.InsertEdgesReply\"\x00\x32\x84\x05\n\x0eUltipaControls\x12\x42\n\x08SayHello\x12\x1a.ultipa.HelloUltipaRequest\x1a\x18.ultipa.HelloUltipaReply\"\x00\x12\x45\n\x0bUserSetting\x12\x1a.ultipa.UserSettingRequest\x1a\x18.ultipa.UserSettingReply\"\x00\x12\x37\n\x07QueryEx\x12\x14.ultipa.QueryRequest\x1a\x12.ultipa.QueryReply\"\x00\x30\x01\x12\x38\n\x06\x45xport\x12\x15.ultipa.ExportRequest\x1a\x13.ultipa.ExportReply\"\x00\x30\x01\x12J\n\x0c\x44ownloadFile\x12\x1b.ultipa.DownloadFileRequest\x1a\x19.ultipa.DownloadFileReply\"\x00\x30\x01\x12G\n\x0bInstallAlgo\x12\x1a.ultipa.InstallAlgoRequest\x1a\x18.ultipa.InstallAlgoReply\"\x00(\x01\x12K\n\rUninstallAlgo\x12\x1c.ultipa.UninstallAlgoRequest\x1a\x1a.ultipa.UninstallAlgoReply\"\x00\x12H\n\x0cRollbackAlgo\x12\x1b.ultipa.RollbackAlgoRequest\x1a\x19.ultipa.RollbackAlgoReply\"\x00\x12H\n\x0c\x41uthenticate\x12\x1b.ultipa.AuthenticateRequest\x1a\x19.ultipa.AuthenticateReply\"\x00\x62\x06proto3')

_QUERYTYPE = DESCRIPTOR.enum_types_by_name['QueryType']
QueryType = enum_type_wrapper.EnumTypeWrapper(_QUERYTYPE)
_RESULTTYPE = DESCRIPTOR.enum_types_by_name['ResultType']
ResultType = enum_type_wrapper.EnumTypeWrapper(_RESULTTYPE)
_INSERTTYPE = DESCRIPTOR.enum_types_by_name['InsertType']
InsertType = enum_type_wrapper.EnumTypeWrapper(_INSERTTYPE)
_PROPERTYTYPE = DESCRIPTOR.enum_types_by_name['PropertyType']
PropertyType = enum_type_wrapper.EnumTypeWrapper(_PROPERTYTYPE)
_ERRORCODE = DESCRIPTOR.enum_types_by_name['ErrorCode']
ErrorCode = enum_type_wrapper.EnumTypeWrapper(_ERRORCODE)
_FOLLOWERROLE = DESCRIPTOR.enum_types_by_name['FollowerRole']
FollowerRole = enum_type_wrapper.EnumTypeWrapper(_FOLLOWERROLE)
_SERVERSTATUS = DESCRIPTOR.enum_types_by_name['ServerStatus']
ServerStatus = enum_type_wrapper.EnumTypeWrapper(_SERVERSTATUS)
_DBTYPE = DESCRIPTOR.enum_types_by_name['DBType']
DBType = enum_type_wrapper.EnumTypeWrapper(_DBTYPE)
_JOB_STATUS = DESCRIPTOR.enum_types_by_name['JOB_STATUS']
JOB_STATUS = enum_type_wrapper.EnumTypeWrapper(_JOB_STATUS)
_AUTHENTICATETYPE = DESCRIPTOR.enum_types_by_name['AuthenticateType']
AuthenticateType = enum_type_wrapper.EnumTypeWrapper(_AUTHENTICATETYPE)
QUERY_UNSET = 0
UQL = 1
GQL = 2
RESULT_TYPE_UNSET = 0
RESULT_TYPE_PATH = 1
RESULT_TYPE_NODE = 2
RESULT_TYPE_EDGE = 3
RESULT_TYPE_ATTR = 4
RESULT_TYPE_TABLE = 5
NORMAL = 0
OVERWRITE = 1
UPSERT = 2
UNSET = 0
INT32 = 1
UINT32 = 2
INT64 = 3
UINT64 = 4
FLOAT = 5
DOUBLE = 6
STRING = 7
DATETIME = 8
TIMESTAMP = 9
TEXT = 10
BLOB = 11
POINT = 12
DECIMAL = 13
LIST = 14
SET = 15
MAP = 16
NULL_ = 17
BOOL = 18
SUCCESS = 0
FAILED = 1
PARAM_ERROR = 2
BASE_DB_ERROR = 3
ENGINE_ERROR = 4
SYSTEM_ERROR = 5
SYNTAX_ERROR = 6
PERMISSION_DENIED = 7
DUPLICATE_ID = 8
RAFT_REDIRECT = 10001
RAFT_LEADER_NOT_YET_ELECTED = 10002
RAFT_LOG_ERROR = 10003
NOT_RAFT_MODE = 10004
RAFT_NO_AVAILABLE_FOLLOWERS = 10005
RAFT_NO_AVAILABLE_ALGO_SERVERS = 10006
ROLE_UNSET = 0
ROLE_READABLE = 1
ROLE_ALGO_EXECUTABLE = 2
ROLE_LEARNER = 4
SERVER_STATUS_UNSET = 0
ALIVE = 1
DEAD = 2
DBNODE = 0
DBEDGE = 1
DBGLOBAL = 2
JOB_PENDING = 0
JOB_COMPUTING = 1
JOB_WRITING = 2
JOB_DONE = 3
JOB_FAILED = 4
JOB_STOPPED = 5
PERMISSION_TYPE_UNSET = 0
PERMISSION_TYPE_UQL = 1
PERMISSION_TYPE_GQL = 2
PERMISSION_TYPE_INSERTNODES = 3
PERMISSION_TYPE_INSERTEDGES = 4
PERMISSION_TYPE_EXPORT = 5
PERMISSION_TYPE_DOWNLOADFILE = 6
PERMISSION_TYPE_INSTALLALGO = 7
PERMISSION_TYPE_UNINSTALLALGO = 8
PERMISSION_TYPE_ROLLBACKALGO = 9


_HELLOULTIPAREQUEST = DESCRIPTOR.message_types_by_name['HelloUltipaRequest']
_HELLOULTIPAREPLY = DESCRIPTOR.message_types_by_name['HelloUltipaReply']
_QUERYREQUEST = DESCRIPTOR.message_types_by_name['QueryRequest']
_RESULTALIAS = DESCRIPTOR.message_types_by_name['ResultAlias']
_QUERYREPLY = DESCRIPTOR.message_types_by_name['QueryReply']
_PATHALIAS = DESCRIPTOR.message_types_by_name['PathAlias']
_EDGEALIAS = DESCRIPTOR.message_types_by_name['EdgeAlias']
_NODEALIAS = DESCRIPTOR.message_types_by_name['NodeAlias']
_ATTRALIAS = DESCRIPTOR.message_types_by_name['AttrAlias']
_ATTR = DESCRIPTOR.message_types_by_name['Attr']
_ATTRLISTDATA = DESCRIPTOR.message_types_by_name['AttrListData']
_ATTRMAPDATA = DESCRIPTOR.message_types_by_name['AttrMapData']
_USERSETTINGREQUEST = DESCRIPTOR.message_types_by_name['UserSettingRequest']
_USERSETTINGREPLY = DESCRIPTOR.message_types_by_name['UserSettingReply']
_DOWNLOADFILEREQUEST = DESCRIPTOR.message_types_by_name['DownloadFileRequest']
_DOWNLOADFILEREPLY = DESCRIPTOR.message_types_by_name['DownloadFileReply']
_EXPORTREQUEST = DESCRIPTOR.message_types_by_name['ExportRequest']
_EXPORTREPLY = DESCRIPTOR.message_types_by_name['ExportReply']
_INSERTNODESREQUEST = DESCRIPTOR.message_types_by_name['InsertNodesRequest']
_INSERTNODESREPLY = DESCRIPTOR.message_types_by_name['InsertNodesReply']
_INSERTEDGESREQUEST = DESCRIPTOR.message_types_by_name['InsertEdgesRequest']
_INSERTEDGESREPLY = DESCRIPTOR.message_types_by_name['InsertEdgesReply']
_WITHSERVER = DESCRIPTOR.message_types_by_name['WithServer']
_INSTALLALGOREQUEST = DESCRIPTOR.message_types_by_name['InstallAlgoRequest']
_INSTALLALGOREPLY = DESCRIPTOR.message_types_by_name['InstallAlgoReply']
_UNINSTALLALGOREQUEST = DESCRIPTOR.message_types_by_name['UninstallAlgoRequest']
_UNINSTALLALGOREPLY = DESCRIPTOR.message_types_by_name['UninstallAlgoReply']
_ROLLBACKALGOREQUEST = DESCRIPTOR.message_types_by_name['RollbackAlgoRequest']
_ROLLBACKALGOREPLY = DESCRIPTOR.message_types_by_name['RollbackAlgoReply']
_STATUS = DESCRIPTOR.message_types_by_name['Status']
_RAFTFOLLOWER = DESCRIPTOR.message_types_by_name['RaftFollower']
_CLUSTERINFO = DESCRIPTOR.message_types_by_name['ClusterInfo']
_PATH = DESCRIPTOR.message_types_by_name['Path']
_TABLE = DESCRIPTOR.message_types_by_name['Table']
_TABLEROW = DESCRIPTOR.message_types_by_name['TableRow']
_ENTITYTABLE = DESCRIPTOR.message_types_by_name['EntityTable']
_ENTITYROW = DESCRIPTOR.message_types_by_name['EntityRow']
_SCHEMA = DESCRIPTOR.message_types_by_name['Schema']
_PROPERTY = DESCRIPTOR.message_types_by_name['Property']
_HEADER = DESCRIPTOR.message_types_by_name['Header']
_VALUE = DESCRIPTOR.message_types_by_name['Value']
_LISTDATA = DESCRIPTOR.message_types_by_name['ListData']
_SETDATA = DESCRIPTOR.message_types_by_name['SetData']
_MAPVALUE = DESCRIPTOR.message_types_by_name['MapValue']
_MAPDATA = DESCRIPTOR.message_types_by_name['MapData']
_EXPLAINPLAN = DESCRIPTOR.message_types_by_name['ExplainPlan']
_PLANNODE = DESCRIPTOR.message_types_by_name['PlanNode']
_AUTHENTICATEREQUEST = DESCRIPTOR.message_types_by_name['AuthenticateRequest']
_AUTHENTICATEREPLY = DESCRIPTOR.message_types_by_name['AuthenticateReply']
_MIGRATIONDATA = DESCRIPTOR.message_types_by_name['MigrationData']
_MIGRATIONDATAARRAY = DESCRIPTOR.message_types_by_name['MigrationDataArray']
_GETLEADERREQUEST = DESCRIPTOR.message_types_by_name['GetLeaderRequest']
_GETLEADERREPLY = DESCRIPTOR.message_types_by_name['GetLeaderReply']
_USERSETTINGREQUEST_OPTION = _USERSETTINGREQUEST.enum_types_by_name['OPTION']
HelloUltipaRequest = _reflection.GeneratedProtocolMessageType('HelloUltipaRequest', (_message.Message,), {
  'DESCRIPTOR' : _HELLOULTIPAREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.HelloUltipaRequest)
  })
_sym_db.RegisterMessage(HelloUltipaRequest)

HelloUltipaReply = _reflection.GeneratedProtocolMessageType('HelloUltipaReply', (_message.Message,), {
  'DESCRIPTOR' : _HELLOULTIPAREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.HelloUltipaReply)
  })
_sym_db.RegisterMessage(HelloUltipaReply)

QueryRequest = _reflection.GeneratedProtocolMessageType('QueryRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.QueryRequest)
  })
_sym_db.RegisterMessage(QueryRequest)

ResultAlias = _reflection.GeneratedProtocolMessageType('ResultAlias', (_message.Message,), {
  'DESCRIPTOR' : _RESULTALIAS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.ResultAlias)
  })
_sym_db.RegisterMessage(ResultAlias)

QueryReply = _reflection.GeneratedProtocolMessageType('QueryReply', (_message.Message,), {
  'DESCRIPTOR' : _QUERYREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.QueryReply)
  })
_sym_db.RegisterMessage(QueryReply)

PathAlias = _reflection.GeneratedProtocolMessageType('PathAlias', (_message.Message,), {
  'DESCRIPTOR' : _PATHALIAS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.PathAlias)
  })
_sym_db.RegisterMessage(PathAlias)

EdgeAlias = _reflection.GeneratedProtocolMessageType('EdgeAlias', (_message.Message,), {
  'DESCRIPTOR' : _EDGEALIAS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.EdgeAlias)
  })
_sym_db.RegisterMessage(EdgeAlias)

NodeAlias = _reflection.GeneratedProtocolMessageType('NodeAlias', (_message.Message,), {
  'DESCRIPTOR' : _NODEALIAS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.NodeAlias)
  })
_sym_db.RegisterMessage(NodeAlias)

AttrAlias = _reflection.GeneratedProtocolMessageType('AttrAlias', (_message.Message,), {
  'DESCRIPTOR' : _ATTRALIAS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.AttrAlias)
  })
_sym_db.RegisterMessage(AttrAlias)

Attr = _reflection.GeneratedProtocolMessageType('Attr', (_message.Message,), {
  'DESCRIPTOR' : _ATTR,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Attr)
  })
_sym_db.RegisterMessage(Attr)

AttrListData = _reflection.GeneratedProtocolMessageType('AttrListData', (_message.Message,), {
  'DESCRIPTOR' : _ATTRLISTDATA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.AttrListData)
  })
_sym_db.RegisterMessage(AttrListData)

AttrMapData = _reflection.GeneratedProtocolMessageType('AttrMapData', (_message.Message,), {
  'DESCRIPTOR' : _ATTRMAPDATA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.AttrMapData)
  })
_sym_db.RegisterMessage(AttrMapData)

UserSettingRequest = _reflection.GeneratedProtocolMessageType('UserSettingRequest', (_message.Message,), {
  'DESCRIPTOR' : _USERSETTINGREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UserSettingRequest)
  })
_sym_db.RegisterMessage(UserSettingRequest)

UserSettingReply = _reflection.GeneratedProtocolMessageType('UserSettingReply', (_message.Message,), {
  'DESCRIPTOR' : _USERSETTINGREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UserSettingReply)
  })
_sym_db.RegisterMessage(UserSettingReply)

DownloadFileRequest = _reflection.GeneratedProtocolMessageType('DownloadFileRequest', (_message.Message,), {
  'DESCRIPTOR' : _DOWNLOADFILEREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.DownloadFileRequest)
  })
_sym_db.RegisterMessage(DownloadFileRequest)

DownloadFileReply = _reflection.GeneratedProtocolMessageType('DownloadFileReply', (_message.Message,), {
  'DESCRIPTOR' : _DOWNLOADFILEREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.DownloadFileReply)
  })
_sym_db.RegisterMessage(DownloadFileReply)

ExportRequest = _reflection.GeneratedProtocolMessageType('ExportRequest', (_message.Message,), {
  'DESCRIPTOR' : _EXPORTREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.ExportRequest)
  })
_sym_db.RegisterMessage(ExportRequest)

ExportReply = _reflection.GeneratedProtocolMessageType('ExportReply', (_message.Message,), {
  'DESCRIPTOR' : _EXPORTREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.ExportReply)
  })
_sym_db.RegisterMessage(ExportReply)

InsertNodesRequest = _reflection.GeneratedProtocolMessageType('InsertNodesRequest', (_message.Message,), {
  'DESCRIPTOR' : _INSERTNODESREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InsertNodesRequest)
  })
_sym_db.RegisterMessage(InsertNodesRequest)

InsertNodesReply = _reflection.GeneratedProtocolMessageType('InsertNodesReply', (_message.Message,), {
  'DESCRIPTOR' : _INSERTNODESREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InsertNodesReply)
  })
_sym_db.RegisterMessage(InsertNodesReply)

InsertEdgesRequest = _reflection.GeneratedProtocolMessageType('InsertEdgesRequest', (_message.Message,), {
  'DESCRIPTOR' : _INSERTEDGESREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InsertEdgesRequest)
  })
_sym_db.RegisterMessage(InsertEdgesRequest)

InsertEdgesReply = _reflection.GeneratedProtocolMessageType('InsertEdgesReply', (_message.Message,), {
  'DESCRIPTOR' : _INSERTEDGESREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InsertEdgesReply)
  })
_sym_db.RegisterMessage(InsertEdgesReply)

WithServer = _reflection.GeneratedProtocolMessageType('WithServer', (_message.Message,), {
  'DESCRIPTOR' : _WITHSERVER,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.WithServer)
  })
_sym_db.RegisterMessage(WithServer)

InstallAlgoRequest = _reflection.GeneratedProtocolMessageType('InstallAlgoRequest', (_message.Message,), {
  'DESCRIPTOR' : _INSTALLALGOREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InstallAlgoRequest)
  })
_sym_db.RegisterMessage(InstallAlgoRequest)

InstallAlgoReply = _reflection.GeneratedProtocolMessageType('InstallAlgoReply', (_message.Message,), {
  'DESCRIPTOR' : _INSTALLALGOREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.InstallAlgoReply)
  })
_sym_db.RegisterMessage(InstallAlgoReply)

UninstallAlgoRequest = _reflection.GeneratedProtocolMessageType('UninstallAlgoRequest', (_message.Message,), {
  'DESCRIPTOR' : _UNINSTALLALGOREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UninstallAlgoRequest)
  })
_sym_db.RegisterMessage(UninstallAlgoRequest)

UninstallAlgoReply = _reflection.GeneratedProtocolMessageType('UninstallAlgoReply', (_message.Message,), {
  'DESCRIPTOR' : _UNINSTALLALGOREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.UninstallAlgoReply)
  })
_sym_db.RegisterMessage(UninstallAlgoReply)

RollbackAlgoRequest = _reflection.GeneratedProtocolMessageType('RollbackAlgoRequest', (_message.Message,), {
  'DESCRIPTOR' : _ROLLBACKALGOREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.RollbackAlgoRequest)
  })
_sym_db.RegisterMessage(RollbackAlgoRequest)

RollbackAlgoReply = _reflection.GeneratedProtocolMessageType('RollbackAlgoReply', (_message.Message,), {
  'DESCRIPTOR' : _ROLLBACKALGOREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.RollbackAlgoReply)
  })
_sym_db.RegisterMessage(RollbackAlgoReply)

Status = _reflection.GeneratedProtocolMessageType('Status', (_message.Message,), {
  'DESCRIPTOR' : _STATUS,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Status)
  })
_sym_db.RegisterMessage(Status)

RaftFollower = _reflection.GeneratedProtocolMessageType('RaftFollower', (_message.Message,), {
  'DESCRIPTOR' : _RAFTFOLLOWER,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.RaftFollower)
  })
_sym_db.RegisterMessage(RaftFollower)

ClusterInfo = _reflection.GeneratedProtocolMessageType('ClusterInfo', (_message.Message,), {
  'DESCRIPTOR' : _CLUSTERINFO,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.ClusterInfo)
  })
_sym_db.RegisterMessage(ClusterInfo)

Path = _reflection.GeneratedProtocolMessageType('Path', (_message.Message,), {
  'DESCRIPTOR' : _PATH,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Path)
  })
_sym_db.RegisterMessage(Path)

Table = _reflection.GeneratedProtocolMessageType('Table', (_message.Message,), {
  'DESCRIPTOR' : _TABLE,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Table)
  })
_sym_db.RegisterMessage(Table)

TableRow = _reflection.GeneratedProtocolMessageType('TableRow', (_message.Message,), {
  'DESCRIPTOR' : _TABLEROW,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.TableRow)
  })
_sym_db.RegisterMessage(TableRow)

EntityTable = _reflection.GeneratedProtocolMessageType('EntityTable', (_message.Message,), {
  'DESCRIPTOR' : _ENTITYTABLE,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.EntityTable)
  })
_sym_db.RegisterMessage(EntityTable)

EntityRow = _reflection.GeneratedProtocolMessageType('EntityRow', (_message.Message,), {
  'DESCRIPTOR' : _ENTITYROW,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.EntityRow)
  })
_sym_db.RegisterMessage(EntityRow)

Schema = _reflection.GeneratedProtocolMessageType('Schema', (_message.Message,), {
  'DESCRIPTOR' : _SCHEMA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Schema)
  })
_sym_db.RegisterMessage(Schema)

Property = _reflection.GeneratedProtocolMessageType('Property', (_message.Message,), {
  'DESCRIPTOR' : _PROPERTY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Property)
  })
_sym_db.RegisterMessage(Property)

Header = _reflection.GeneratedProtocolMessageType('Header', (_message.Message,), {
  'DESCRIPTOR' : _HEADER,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Header)
  })
_sym_db.RegisterMessage(Header)

Value = _reflection.GeneratedProtocolMessageType('Value', (_message.Message,), {
  'DESCRIPTOR' : _VALUE,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.Value)
  })
_sym_db.RegisterMessage(Value)

ListData = _reflection.GeneratedProtocolMessageType('ListData', (_message.Message,), {
  'DESCRIPTOR' : _LISTDATA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.ListData)
  })
_sym_db.RegisterMessage(ListData)

SetData = _reflection.GeneratedProtocolMessageType('SetData', (_message.Message,), {
  'DESCRIPTOR' : _SETDATA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.SetData)
  })
_sym_db.RegisterMessage(SetData)

MapValue = _reflection.GeneratedProtocolMessageType('MapValue', (_message.Message,), {
  'DESCRIPTOR' : _MAPVALUE,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.MapValue)
  })
_sym_db.RegisterMessage(MapValue)

MapData = _reflection.GeneratedProtocolMessageType('MapData', (_message.Message,), {
  'DESCRIPTOR' : _MAPDATA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.MapData)
  })
_sym_db.RegisterMessage(MapData)

ExplainPlan = _reflection.GeneratedProtocolMessageType('ExplainPlan', (_message.Message,), {
  'DESCRIPTOR' : _EXPLAINPLAN,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.ExplainPlan)
  })
_sym_db.RegisterMessage(ExplainPlan)

PlanNode = _reflection.GeneratedProtocolMessageType('PlanNode', (_message.Message,), {
  'DESCRIPTOR' : _PLANNODE,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.PlanNode)
  })
_sym_db.RegisterMessage(PlanNode)

AuthenticateRequest = _reflection.GeneratedProtocolMessageType('AuthenticateRequest', (_message.Message,), {
  'DESCRIPTOR' : _AUTHENTICATEREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.AuthenticateRequest)
  })
_sym_db.RegisterMessage(AuthenticateRequest)

AuthenticateReply = _reflection.GeneratedProtocolMessageType('AuthenticateReply', (_message.Message,), {
  'DESCRIPTOR' : _AUTHENTICATEREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.AuthenticateReply)
  })
_sym_db.RegisterMessage(AuthenticateReply)

MigrationData = _reflection.GeneratedProtocolMessageType('MigrationData', (_message.Message,), {
  'DESCRIPTOR' : _MIGRATIONDATA,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.MigrationData)
  })
_sym_db.RegisterMessage(MigrationData)

MigrationDataArray = _reflection.GeneratedProtocolMessageType('MigrationDataArray', (_message.Message,), {
  'DESCRIPTOR' : _MIGRATIONDATAARRAY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.MigrationDataArray)
  })
_sym_db.RegisterMessage(MigrationDataArray)

GetLeaderRequest = _reflection.GeneratedProtocolMessageType('GetLeaderRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETLEADERREQUEST,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.GetLeaderRequest)
  })
_sym_db.RegisterMessage(GetLeaderRequest)

GetLeaderReply = _reflection.GeneratedProtocolMessageType('GetLeaderReply', (_message.Message,), {
  'DESCRIPTOR' : _GETLEADERREPLY,
  '__module__' : 'ultipa_pb2'
  # @@protoc_insertion_point(class_scope:ultipa.GetLeaderReply)
  })
_sym_db.RegisterMessage(GetLeaderReply)

_ULTIPARPCS = DESCRIPTOR.services_by_name['UltipaRpcs']
_ULTIPACONTROLS = DESCRIPTOR.services_by_name['UltipaControls']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _INSERTNODESREPLY.fields_by_name['uuids']._options = None
  _INSERTNODESREPLY.fields_by_name['uuids']._serialized_options = b'0\001'
  _INSERTEDGESREPLY.fields_by_name['uuids']._options = None
  _INSERTEDGESREPLY.fields_by_name['uuids']._serialized_options = b'0\001'
  _ENTITYROW.fields_by_name['uuid']._options = None
  _ENTITYROW.fields_by_name['uuid']._serialized_options = b'0\001'
  _ENTITYROW.fields_by_name['from_uuid']._options = None
  _ENTITYROW.fields_by_name['from_uuid']._serialized_options = b'0\001'
  _ENTITYROW.fields_by_name['to_uuid']._options = None
  _ENTITYROW.fields_by_name['to_uuid']._serialized_options = b'0\001'
  _QUERYTYPE._serialized_start=5183
  _QUERYTYPE._serialized_end=5229
  _RESULTTYPE._serialized_start=5232
  _RESULTTYPE._serialized_end=5378
  _INSERTTYPE._serialized_start=5380
  _INSERTTYPE._serialized_end=5431
  _PROPERTYTYPE._serialized_start=5434
  _PROPERTYTYPE._serialized_end=5662
  _ERRORCODE._serialized_start=5665
  _ERRORCODE._serialized_end=5998
  _FOLLOWERROLE._serialized_start=6000
  _FOLLOWERROLE._serialized_end=6093
  _SERVERSTATUS._serialized_start=6095
  _SERVERSTATUS._serialized_end=6155
  _DBTYPE._serialized_start=6157
  _DBTYPE._serialized_end=6203
  _JOB_STATUS._serialized_start=6205
  _JOB_STATUS._serialized_end=6317
  _AUTHENTICATETYPE._serialized_start=6320
  _AUTHENTICATETYPE._serialized_end=6645
  _HELLOULTIPAREQUEST._serialized_start=24
  _HELLOULTIPAREQUEST._serialized_end=58
  _HELLOULTIPAREPLY._serialized_start=60
  _HELLOULTIPAREPLY._serialized_end=127
  _QUERYREQUEST._serialized_start=130
  _QUERYREQUEST._serialized_end=310
  _RESULTALIAS._serialized_start=312
  _RESULTALIAS._serialized_end=381
  _QUERYREPLY._serialized_start=384
  _QUERYREPLY._serialized_end=760
  _PATHALIAS._serialized_start=762
  _PATHALIAS._serialized_end=817
  _EDGEALIAS._serialized_start=819
  _EDGEALIAS._serialized_end=886
  _NODEALIAS._serialized_start=888
  _NODEALIAS._serialized_end=955
  _ATTRALIAS._serialized_start=957
  _ATTRALIAS._serialized_end=1011
  _ATTR._serialized_start=1013
  _ATTR._serialized_end=1077
  _ATTRLISTDATA._serialized_start=1080
  _ATTRLISTDATA._serialized_end=1275
  _ATTRMAPDATA._serialized_start=1277
  _ATTRMAPDATA._serialized_end=1363
  _USERSETTINGREQUEST._serialized_start=1366
  _USERSETTINGREQUEST._serialized_end=1517
  _USERSETTINGREQUEST_OPTION._serialized_start=1483
  _USERSETTINGREQUEST_OPTION._serialized_end=1517
  _USERSETTINGREPLY._serialized_start=1519
  _USERSETTINGREPLY._serialized_end=1583
  _DOWNLOADFILEREQUEST._serialized_start=1585
  _DOWNLOADFILEREQUEST._serialized_end=1625
  _DOWNLOADFILEREPLY._serialized_start=1627
  _DOWNLOADFILEREPLY._serialized_end=1713
  _EXPORTREQUEST._serialized_start=1715
  _EXPORTREQUEST._serialized_end=1836
  _EXPORTREPLY._serialized_start=1838
  _EXPORTREPLY._serialized_end=1965
  _INSERTNODESREQUEST._serialized_start=1968
  _INSERTNODESREQUEST._serialized_end=2162
  _INSERTNODESREPLY._serialized_start=2165
  _INSERTNODESREPLY._serialized_end=2379
  _INSERTEDGESREQUEST._serialized_start=2382
  _INSERTEDGESREQUEST._serialized_end=2592
  _INSERTEDGESREPLY._serialized_start=2595
  _INSERTEDGESREPLY._serialized_end=2796
  _WITHSERVER._serialized_start=2798
  _WITHSERVER._serialized_end=2835
  _INSTALLALGOREQUEST._serialized_start=2837
  _INSTALLALGOREQUEST._serialized_end=2945
  _INSTALLALGOREPLY._serialized_start=2947
  _INSTALLALGOREPLY._serialized_end=2997
  _UNINSTALLALGOREQUEST._serialized_start=2999
  _UNINSTALLALGOREQUEST._serialized_end=3081
  _UNINSTALLALGOREPLY._serialized_start=3083
  _UNINSTALLALGOREPLY._serialized_end=3135
  _ROLLBACKALGOREQUEST._serialized_start=3137
  _ROLLBACKALGOREQUEST._serialized_end=3218
  _ROLLBACKALGOREPLY._serialized_start=3220
  _ROLLBACKALGOREPLY._serialized_end=3271
  _STATUS._serialized_start=3273
  _STATUS._serialized_end=3376
  _RAFTFOLLOWER._serialized_start=3378
  _RAFTFOLLOWER._serialized_end=3485
  _CLUSTERINFO._serialized_start=3487
  _CLUSTERINFO._serialized_end=3614
  _PATH._serialized_start=3616
  _PATH._serialized_end=3721
  _TABLE._serialized_start=3723
  _TABLE._serialized_end=3821
  _TABLEROW._serialized_start=3823
  _TABLEROW._serialized_end=3849
  _ENTITYTABLE._serialized_start=3851
  _ENTITYTABLE._serialized_end=3937
  _ENTITYROW._serialized_start=3940
  _ENTITYROW._serialized_end=4176
  _SCHEMA._serialized_start=4178
  _SCHEMA._serialized_end=4264
  _PROPERTY._serialized_start=4267
  _PROPERTY._serialized_end=4407
  _HEADER._serialized_start=4409
  _HEADER._serialized_end=4485
  _VALUE._serialized_start=4487
  _VALUE._serialized_end=4522
  _LISTDATA._serialized_start=4524
  _LISTDATA._serialized_end=4567
  _SETDATA._serialized_start=4569
  _SETDATA._serialized_end=4611
  _MAPVALUE._serialized_start=4613
  _MAPVALUE._serialized_end=4651
  _MAPDATA._serialized_start=4653
  _MAPDATA._serialized_end=4713
  _EXPLAINPLAN._serialized_start=4715
  _EXPLAINPLAN._serialized_end=4766
  _PLANNODE._serialized_start=4768
  _PLANNODE._serialized_end=4850
  _AUTHENTICATEREQUEST._serialized_start=4852
  _AUTHENTICATEREQUEST._serialized_end=4933
  _AUTHENTICATEREPLY._serialized_start=4935
  _AUTHENTICATEREPLY._serialized_end=4986
  _MIGRATIONDATA._serialized_start=4988
  _MIGRATIONDATA._serialized_end=5031
  _MIGRATIONDATAARRAY._serialized_start=5033
  _MIGRATIONDATAARRAY._serialized_end=5091
  _GETLEADERREQUEST._serialized_start=5093
  _GETLEADERREQUEST._serialized_end=5131
  _GETLEADERREPLY._serialized_start=5133
  _GETLEADERREPLY._serialized_end=5181
  _ULTIPARPCS._serialized_start=6648
  _ULTIPARPCS._serialized_end=6925
  _ULTIPACONTROLS._serialized_start=6928
  _ULTIPACONTROLS._serialized_end=7572
# @@protoc_insertion_point(module_scope)
